@@ -14,14 +14,14 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/be/src/exec/hash-table.h
+// and modified by Doris
 
-#ifndef DORIS_BE_SRC_QUERY_EXEC_HASH_TABLE_H
-#define DORIS_BE_SRC_QUERY_EXEC_HASH_TABLE_H
+#pragma once
 
 #include <vector>
 
-#include "common/logging.h"
-#include "common/object_pool.h"
 #include "common/status.h"
 #include "util/hash_util.hpp"
 
@@ -71,6 +71,7 @@ class RuntimeState;
 // all the rows and then calls scan to find them.  Aggregation interleaves find() and
 // inserts().  We can want to optimize joins more heavily for inserts() (in particular
 // growing).
+
 class HashTable {
 private:
     struct Node;
@@ -89,8 +90,7 @@ public:
     //  - initial_seed: Initial seed value to use when computing hashes for rows
     HashTable(const std::vector<ExprContext*>& build_exprs,
               const std::vector<ExprContext*>& probe_exprs, int num_build_tuples, bool stores_nulls,
-              const std::vector<bool>& finds_nulls, int32_t initial_seed,
-              const std::shared_ptr<MemTracker>& mem_tracker, int64_t num_buckets);
+              const std::vector<bool>& finds_nulls, int32_t initial_seed, int64_t num_buckets);
 
     ~HashTable();
 
@@ -149,22 +149,19 @@ public:
     Iterator find(TupleRow* probe_row, bool probe = true);
 
     // Returns number of elements in the hash table
-    int64_t size() { return _num_nodes; }
+    int64_t size() const { return _num_nodes; }
 
     // Returns the number of buckets
-    int64_t num_buckets() { return _buckets.size(); }
+    int64_t num_buckets() const { return _buckets.size(); }
 
     // Returns the number of filled buckets
-    int64_t num_filled_buckets() { return _num_filled_buckets; }
+    int64_t num_filled_buckets() const { return _num_filled_buckets; }
 
     // Check the hash table should be shrink
     bool should_be_shrink(int64_t valid_row) {
         return valid_row < MAX_BUCKET_OCCUPANCY_FRACTION * (_buckets.size() / 2.0);
     }
 
-    // true if any of the MemTrackers was exceeded
-    bool exceeded_limit() const { return _exceeded_limit; }
-
     // Returns the load factor (the number of non-empty buckets)
     float load_factor() { return _num_filled_buckets / static_cast<float>(_buckets.size()); }
 
@@ -197,7 +194,9 @@ public:
     // just the build row addresses.
     std::string debug_string(bool skip_empty, const RowDescriptor* build_desc);
 
-    inline std::pair<int64_t, int64_t> minmax_node();
+    std::pair<int64_t, int64_t> minmax_node();
+
+    MemTracker* mem_tracker() { return _mem_tracker.get(); }
 
     // Load factor that will trigger growing the hash table on insert.  This is
     // defined as the number of non-empty buckets / total_buckets
@@ -212,7 +211,51 @@ public:
         // from a Find, this will lazily evaluate that bucket, only returning
         // TupleRows that match the current scan row.
         template <bool check_match>
-        void next();
+        void next() {
+            if (_bucket_idx == -1) {
+                return;
+            }
+
+            // TODO: this should prefetch the next tuplerow
+            Node* node = _node;
+
+            // Iterator is not from a full table scan, evaluate equality now.  Only the current
+            // bucket needs to be scanned. '_expr_values_buffer' contains the results
+            // for the current probe row.
+            if (check_match) {
+                // TODO: this should prefetch the next node
+                Node* next_node = node->_next;
+
+                while (next_node != nullptr) {
+                    node = next_node;
+
+                    if (node->_hash == _scan_hash && _table->equals(node->data())) {
+                        _node = next_node;
+                        return;
+                    }
+
+                    next_node = node->_next;
+                }
+
+                *this = _table->end();
+            } else {
+                // Move onto the next chained node
+                if (node->_next != nullptr) {
+                    _node = node->_next;
+                    return;
+                }
+
+                // Move onto the next bucket
+                Bucket* bucket = _table->next_bucket(&_bucket_idx);
+
+                if (bucket == nullptr) {
+                    _bucket_idx = -1;
+                    _node = nullptr;
+                } else {
+                    _node = bucket->_node;
+                }
+            }
+        }
 
         // Returns the current row or nullptr if at end.
         TupleRow* get_row() {
@@ -243,11 +286,11 @@ public:
             return _node->matched;
         }
 
-        bool operator==(const Iterator& rhs) {
+        bool operator==(const Iterator& rhs) const {
             return _bucket_idx == rhs._bucket_idx && _node == rhs._node;
         }
 
-        bool operator!=(const Iterator& rhs) {
+        bool operator!=(const Iterator& rhs) const {
             return _bucket_idx != rhs._bucket_idx || _node != rhs._node;
         }
 
@@ -372,11 +415,6 @@ private:
     // Grow the node array.
     void grow_node_array();
 
-    // Sets _mem_tracker_exceeded to true and MEM_LIMIT_EXCEEDED for the query.
-    // allocation_size is the attempted size of the allocation that would have
-    // brought us over the mem limit.
-    void mem_limit_exceeded(int64_t allocation_size);
-
     const std::vector<ExprContext*>& _build_expr_ctxs;
     const std::vector<ExprContext*>& _probe_expr_ctxs;
 
@@ -406,9 +444,7 @@ private:
     // total capacity
     int64_t _total_capacity;
 
-    bool _exceeded_limit; // true if any of _mem_trackers[].limit_exceeded()
-
-    std::shared_ptr<MemTracker> _mem_tracker;
+    std::unique_ptr<MemTracker> _mem_tracker;
 
     std::vector<Bucket> _buckets;
 
@@ -442,5 +478,3 @@ private:
 };
 
 } // namespace doris
-
-#endif
