@@ -89,14 +89,76 @@ struct CpuMetrics {
 #define DEFINE_MEMORY_GAUGE_METRIC(metric, unit) \
     DEFINE_GAUGE_METRIC_PROTOTYPE_2ARG(memory_##metric, unit);
 DEFINE_MEMORY_GAUGE_METRIC(allocated_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(pgpgin, MetricUnit::NOUNIT);
+DEFINE_MEMORY_GAUGE_METRIC(pgpgout, MetricUnit::NOUNIT);
+DEFINE_MEMORY_GAUGE_METRIC(pswpin, MetricUnit::NOUNIT);
+DEFINE_MEMORY_GAUGE_METRIC(pswpout, MetricUnit::NOUNIT);
+#ifndef USE_JEMALLOC
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_allocated_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_total_thread_cache_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_central_cache_free_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_transfer_cache_free_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_thread_cache_free_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_pageheap_free_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(tcmalloc_pageheap_unmapped_bytes, MetricUnit::BYTES);
+#else
+DEFINE_MEMORY_GAUGE_METRIC(jemalloc_allocated_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(jemalloc_active_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(jemalloc_metadata_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(jemalloc_resident_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(jemalloc_mapped_bytes, MetricUnit::BYTES);
+DEFINE_MEMORY_GAUGE_METRIC(jemalloc_retained_bytes, MetricUnit::BYTES);
+#endif
 
 struct MemoryMetrics {
     MemoryMetrics(MetricEntity* ent) : entity(ent) {
         INT_GAUGE_METRIC_REGISTER(entity, memory_allocated_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_pgpgin);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_pgpgout);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_pswpin);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_pswpout);
+
+#ifndef USE_JEMALLOC
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_allocated_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_total_thread_cache_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_central_cache_free_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_transfer_cache_free_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_thread_cache_free_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_pageheap_free_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_tcmalloc_pageheap_unmapped_bytes);
+#else
+        INT_GAUGE_METRIC_REGISTER(entity, memory_jemalloc_allocated_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_jemalloc_active_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_jemalloc_metadata_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_jemalloc_resident_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_jemalloc_mapped_bytes);
+        INT_GAUGE_METRIC_REGISTER(entity, memory_jemalloc_retained_bytes);
+#endif
     }
 
     MetricEntity* entity = nullptr;
     IntGauge* memory_allocated_bytes;
+    IntGauge* memory_pgpgin;
+    IntGauge* memory_pgpgout;
+    IntGauge* memory_pswpin;
+    IntGauge* memory_pswpout;
+
+#ifndef USE_JEMALLOC
+    IntGauge* memory_tcmalloc_allocated_bytes;
+    IntGauge* memory_tcmalloc_total_thread_cache_bytes;
+    IntGauge* memory_tcmalloc_central_cache_free_bytes;
+    IntGauge* memory_tcmalloc_transfer_cache_free_bytes;
+    IntGauge* memory_tcmalloc_thread_cache_free_bytes;
+    IntGauge* memory_tcmalloc_pageheap_free_bytes;
+    IntGauge* memory_tcmalloc_pageheap_unmapped_bytes;
+#else
+    IntGauge* memory_jemalloc_allocated_bytes;
+    IntGauge* memory_jemalloc_active_bytes;
+    IntGauge* memory_jemalloc_metadata_bytes;
+    IntGauge* memory_jemalloc_resident_bytes;
+    IntGauge* memory_jemalloc_mapped_bytes;
+    IntGauge* memory_jemalloc_retained_bytes;
+#endif
 };
 
 #define DEFINE_DISK_COUNTER_METRIC(metric, unit) \
@@ -215,6 +277,34 @@ struct LoadAverageMetrics {
     DoubleGauge* load_average_15_minutes;
 };
 
+#define DEFINE_PROC_STAT_COUNTER_METRIC(metric)                                       \
+    DEFINE_COUNTER_METRIC_PROTOTYPE_5ARG(proc_##metric, MetricUnit::NOUNIT, "", proc, \
+                                         Labels({{"mode", #metric}}));
+DEFINE_PROC_STAT_COUNTER_METRIC(interrupt);
+DEFINE_PROC_STAT_COUNTER_METRIC(ctxt_switch);
+DEFINE_PROC_STAT_COUNTER_METRIC(procs_running);
+DEFINE_PROC_STAT_COUNTER_METRIC(procs_blocked);
+
+struct ProcMetrics {
+    ProcMetrics(MetricEntity* ent) : entity(ent) {
+        INT_ATOMIC_COUNTER_METRIC_REGISTER(entity, proc_interrupt);
+        INT_ATOMIC_COUNTER_METRIC_REGISTER(entity, proc_ctxt_switch);
+        INT_ATOMIC_COUNTER_METRIC_REGISTER(entity, proc_procs_running);
+        INT_ATOMIC_COUNTER_METRIC_REGISTER(entity, proc_procs_blocked);
+    }
+
+    MetricEntity* entity = nullptr;
+
+    IntAtomicCounter* proc_interrupt;
+    IntAtomicCounter* proc_ctxt_switch;
+    IntAtomicCounter* proc_procs_running;
+    IntAtomicCounter* proc_procs_blocked;
+};
+
+DEFINE_GAUGE_CORE_METRIC_PROTOTYPE_2ARG(max_disk_io_util_percent, MetricUnit::PERCENT);
+DEFINE_GAUGE_CORE_METRIC_PROTOTYPE_2ARG(max_network_send_bytes_rate, MetricUnit::BYTES);
+DEFINE_GAUGE_CORE_METRIC_PROTOTYPE_2ARG(max_network_receive_bytes_rate, MetricUnit::BYTES);
+
 const char* SystemMetrics::_s_hook_name = "system_metrics";
 
 SystemMetrics::SystemMetrics(MetricRegistry* registry, const std::set<std::string>& disk_devices,
@@ -224,19 +314,27 @@ SystemMetrics::SystemMetrics(MetricRegistry* registry, const std::set<std::strin
     _server_entity = _registry->register_entity("server");
     DCHECK(_server_entity != nullptr);
     _server_entity->register_hook(_s_hook_name, std::bind(&SystemMetrics::update, this));
-    _install_cpu_metrics(_server_entity.get());
+    _install_cpu_metrics();
     _install_memory_metrics(_server_entity.get());
     _install_disk_metrics(disk_devices);
     _install_net_metrics(network_interfaces);
     _install_fd_metrics(_server_entity.get());
     _install_snmp_metrics(_server_entity.get());
     _install_load_avg_metrics(_server_entity.get());
+    _install_proc_metrics(_server_entity.get());
+
+    INT_GAUGE_METRIC_REGISTER(_server_entity.get(), max_disk_io_util_percent);
+    INT_GAUGE_METRIC_REGISTER(_server_entity.get(), max_network_send_bytes_rate);
+    INT_GAUGE_METRIC_REGISTER(_server_entity.get(), max_network_receive_bytes_rate);
 }
 
 SystemMetrics::~SystemMetrics() {
     DCHECK(_server_entity != nullptr);
     _server_entity->deregister_hook(_s_hook_name);
 
+    for (auto& it : _cpu_metrics) {
+        delete it.second;
+    }
     for (auto& it : _disk_metrics) {
         delete it.second;
     }
@@ -256,10 +354,16 @@ void SystemMetrics::update() {
     _update_fd_metrics();
     _update_snmp_metrics();
     _update_load_avg_metrics();
+    _update_proc_metrics();
 }
 
-void SystemMetrics::_install_cpu_metrics(MetricEntity* entity) {
-    _cpu_metrics.reset(new CpuMetrics(entity));
+void SystemMetrics::_install_cpu_metrics() {
+    get_cpu_name();
+    for (auto cpu_name : _cpu_names) {
+        auto cpu_entity = _registry->register_entity(cpu_name, {{"device", cpu_name}});
+        CpuMetrics* metrics = new CpuMetrics(cpu_entity.get());
+        _cpu_metrics.emplace(cpu_name, metrics);
+    }
 }
 
 #ifdef BE_TEST
@@ -269,6 +373,7 @@ const char* k_ut_net_dev_path;
 const char* k_ut_fd_path;
 const char* k_ut_net_snmp_path;
 const char* k_ut_load_avg_path;
+const char* k_ut_vmstat_path;
 #endif
 
 void SystemMetrics::_update_cpu_metrics() {
@@ -284,26 +389,35 @@ void SystemMetrics::_update_cpu_metrics() {
         return;
     }
 
-    if (getline(&_line_ptr, &_line_buf_size, fp) < 0) {
+    while (getline(&_line_ptr, &_line_buf_size, fp) > 0) {
+        char cpu[16];
+        int64_t values[CpuMetrics::cpu_num_metrics];
+        memset(values, 0, sizeof(values));
+        int num = sscanf(_line_ptr,
+                         "%15s"
+                         " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64
+                         " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64,
+                         cpu, &values[0], &values[1], &values[2], &values[3], &values[4],
+                         &values[5], &values[6], &values[7], &values[8], &values[9]);
+        if (num < 4) {
+            continue;
+        }
+
+        std::string cpu_name(cpu);
+        auto it = _cpu_metrics.find(cpu_name);
+        if (it == _cpu_metrics.end()) {
+            continue;
+        }
+
+        for (int i = 0; i < CpuMetrics::cpu_num_metrics; ++i) {
+            it->second->metrics[i]->set_value(values[i]);
+        }
+    }
+
+    if (ferror(fp) != 0) {
         char buf[64];
         LOG(WARNING) << "getline failed, errno=" << errno
                      << ", message=" << strerror_r(errno, buf, 64);
-        fclose(fp);
-        return;
-    }
-
-    char cpu[16];
-    int64_t values[CpuMetrics::cpu_num_metrics];
-    memset(values, 0, sizeof(values));
-    sscanf(_line_ptr,
-           "%15s"
-           " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64
-           " %" PRId64 " %" PRId64 " %" PRId64,
-           cpu, &values[0], &values[1], &values[2], &values[3], &values[4], &values[5], &values[6],
-           &values[7], &values[8], &values[9]);
-
-    for (int i = 0; i < CpuMetrics::cpu_num_metrics; ++i) {
-        _cpu_metrics->metrics[i]->set_value(values[i]);
     }
 
     fclose(fp);
@@ -315,6 +429,41 @@ void SystemMetrics::_install_memory_metrics(MetricEntity* entity) {
 
 void SystemMetrics::_update_memory_metrics() {
     _memory_metrics->memory_allocated_bytes->set_value(PerfCounters::get_vm_rss());
+    get_metrics_from_proc_vmstat();
+}
+
+void SystemMetrics::update_allocator_metrics() {
+#if defined(ADDRESS_SANITIZER) || defined(LEAK_SANITIZER) || defined(THREAD_SANITIZER)
+    LOG(INFO) << "Memory tracking is not available with address sanitizer builds.";
+#elif defined(USE_JEMALLOC)
+    _memory_metrics->memory_jemalloc_allocated_bytes->set_value(
+            MemInfo::get_je_metrics("stats.allocated"));
+    _memory_metrics->memory_jemalloc_active_bytes->set_value(
+            MemInfo::get_je_metrics("stats.active"));
+    _memory_metrics->memory_jemalloc_metadata_bytes->set_value(
+            MemInfo::get_je_metrics("stats.metadata"));
+    _memory_metrics->memory_jemalloc_resident_bytes->set_value(
+            MemInfo::get_je_metrics("stats.resident"));
+    _memory_metrics->memory_jemalloc_mapped_bytes->set_value(
+            MemInfo::get_je_metrics("stats.mapped"));
+    _memory_metrics->memory_jemalloc_retained_bytes->set_value(
+            MemInfo::get_je_metrics("stats.retained"));
+#else
+    _memory_metrics->memory_tcmalloc_allocated_bytes->set_value(
+            MemInfo::get_tc_metrics("generic.total_physical_bytes"));
+    _memory_metrics->memory_tcmalloc_total_thread_cache_bytes->set_value(
+            MemInfo::allocator_cache_mem());
+    _memory_metrics->memory_tcmalloc_central_cache_free_bytes->set_value(
+            MemInfo::get_tc_metrics("tcmalloc.central_cache_free_bytes"));
+    _memory_metrics->memory_tcmalloc_transfer_cache_free_bytes->set_value(
+            MemInfo::get_tc_metrics("tcmalloc.transfer_cache_free_bytes"));
+    _memory_metrics->memory_tcmalloc_thread_cache_free_bytes->set_value(
+            MemInfo::get_tc_metrics("tcmalloc.thread_cache_free_bytes"));
+    _memory_metrics->memory_tcmalloc_pageheap_free_bytes->set_value(
+            MemInfo::get_tc_metrics("tcmalloc.pageheap_free_bytes"));
+    _memory_metrics->memory_tcmalloc_pageheap_unmapped_bytes->set_value(
+            MemInfo::get_tc_metrics("tcmalloc.pageheap_unmapped_bytes"));
+#endif
 }
 
 void SystemMetrics::_install_disk_metrics(const std::set<std::string>& disk_devices) {
@@ -719,4 +868,146 @@ void SystemMetrics::get_max_net_traffic(const std::map<std::string, int64_t>& ls
     *send_rate = max_send / interval_sec;
     *rcv_rate = max_rcv / interval_sec;
 }
+
+void SystemMetrics::update_max_disk_io_util_percent(const std::map<std::string, int64_t>& lst_value,
+                                                    int64_t interval_sec) {
+    max_disk_io_util_percent->set_value(get_max_io_util(lst_value, interval_sec));
+}
+
+void SystemMetrics::update_max_network_send_bytes_rate(int64_t max_send_bytes_rate) {
+    max_network_send_bytes_rate->set_value(max_send_bytes_rate);
+}
+
+void SystemMetrics::update_max_network_receive_bytes_rate(int64_t max_receive_bytes_rate) {
+    max_network_receive_bytes_rate->set_value(max_receive_bytes_rate);
+}
+
+void SystemMetrics::_install_proc_metrics(MetricEntity* entity) {
+    _proc_metrics.reset(new ProcMetrics(entity));
+}
+
+void SystemMetrics::_update_proc_metrics() {
+#ifdef BE_TEST
+    FILE* fp = fopen(k_ut_stat_path, "r");
+#else
+    FILE* fp = fopen("/proc/stat", "r");
+#endif
+    if (fp == nullptr) {
+        char buf[64];
+        LOG(WARNING) << "open /proc/stat failed, errno=" << errno
+                     << ", message=" << strerror_r(errno, buf, 64);
+        return;
+    }
+
+    uint64_t inter = 0, ctxt = 0, procs_r = 0, procs_b = 0;
+    while (getline(&_line_ptr, &_line_buf_size, fp) > 0) {
+        char* start_pos = nullptr;
+        start_pos = strstr(_line_ptr, "intr ");
+        if (start_pos) {
+            sscanf(start_pos, "intr %" PRIu64, &inter);
+            _proc_metrics->proc_interrupt->set_value(inter);
+        }
+
+        start_pos = strstr(_line_ptr, "ctxt ");
+        if (start_pos) {
+            sscanf(start_pos, "ctxt %" PRIu64, &ctxt);
+            _proc_metrics->proc_ctxt_switch->set_value(ctxt);
+        }
+
+        start_pos = strstr(_line_ptr, "procs_running ");
+        if (start_pos) {
+            sscanf(start_pos, "procs_running %" PRIu64, &procs_r);
+            _proc_metrics->proc_procs_running->set_value(procs_r);
+        }
+
+        start_pos = strstr(_line_ptr, "procs_blocked ");
+        if (start_pos) {
+            sscanf(start_pos, "procs_blocked %" PRIu64, &procs_b);
+            _proc_metrics->proc_procs_blocked->set_value(procs_b);
+        }
+    }
+
+    if (ferror(fp) != 0) {
+        char buf[64];
+        LOG(WARNING) << "getline failed, errno=" << errno
+                     << ", message=" << strerror_r(errno, buf, 64);
+    }
+
+    fclose(fp);
+}
+
+void SystemMetrics::get_metrics_from_proc_vmstat() {
+#ifdef BE_TEST
+    FILE* fp = fopen(k_ut_vmstat_path, "r");
+#else
+    FILE* fp = fopen("/proc/vmstat", "r");
+#endif
+    if (fp == nullptr) {
+        char buf[64];
+        LOG(WARNING) << "open /proc/vmstat failed, errno=" << errno
+                     << ", message=" << strerror_r(errno, buf, 64);
+        return;
+    }
+
+    while (getline(&_line_ptr, &_line_buf_size, fp) > 0) {
+        uint64_t value;
+        char name[64];
+        int num = sscanf(_line_ptr, "%s %" PRIu64, name, &value);
+        if (num < 2) {
+            continue;
+        }
+
+        if (strcmp(name, "pgpgin") == 0) {
+            _memory_metrics->memory_pgpgin->set_value(value);
+        } else if (strcmp(name, "pgpgout") == 0) {
+            _memory_metrics->memory_pgpgout->set_value(value);
+        } else if (strcmp(name, "pswpin") == 0) {
+            _memory_metrics->memory_pswpin->set_value(value);
+        } else if (strcmp(name, "pswpout") == 0) {
+            _memory_metrics->memory_pswpout->set_value(value);
+        }
+    }
+
+    if (ferror(fp) != 0) {
+        char buf[64];
+        LOG(WARNING) << "getline failed, errno=" << errno
+                     << ", message=" << strerror_r(errno, buf, 64);
+    }
+
+    fclose(fp);
+}
+
+void SystemMetrics::get_cpu_name() {
+#ifdef BE_TEST
+    FILE* fp = fopen(k_ut_stat_path, "r");
+#else
+    FILE* fp = fopen("/proc/stat", "r");
+#endif
+    if (fp == nullptr) {
+        char buf[64];
+        LOG(WARNING) << "open /proc/stat failed, errno=" << errno
+                     << ", message=" << strerror_r(errno, buf, 64);
+        return;
+    }
+
+    while (getline(&_line_ptr, &_line_buf_size, fp) > 0) {
+        char cpu[16];
+        char* start_pos = nullptr;
+        start_pos = strstr(_line_ptr, "cpu");
+        if (start_pos) {
+            sscanf(_line_ptr, "%15s", cpu);
+            std::string cpu_name(cpu);
+            _cpu_names.push_back(cpu_name);
+        }
+    }
+
+    if (ferror(fp) != 0) {
+        char buf[64];
+        LOG(WARNING) << "getline failed, errno=" << errno
+                     << ", message=" << strerror_r(errno, buf, 64);
+    }
+
+    fclose(fp);
+}
+
 } // namespace doris
