@@ -24,10 +24,9 @@
 #include <string_view>
 
 #include "runtime/string_search.hpp"
-#include "util/encryption_util.h"
-#include "util/simd/vstring_function.h"
 #include "util/url_coding.h"
 #include "vec/common/pod_array_fwd.h"
+#include "vec/functions/function_reverse.h"
 #include "vec/functions/function_string_to_string.h"
 #include "vec/functions/function_totype.h"
 #include "vec/functions/simple_function_factory.h"
@@ -49,8 +48,7 @@ struct StringASCII {
         res.resize(size);
         for (int i = 0; i < size; ++i) {
             const char* raw_str = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            // if strlen(raw_str) == 0, raw_str[0] is '\0'
-            res[i] = (strlen(raw_str) == 0) ? 0 : static_cast<uint8_t>(raw_str[0]);
+            res[i] = (offsets[i] == offsets[i - 1]) ? 0 : static_cast<uint8_t>(raw_str[0]);
         }
         return Status::OK();
     }
@@ -71,7 +69,7 @@ struct StringLengthImpl {
         auto size = offsets.size();
         res.resize(size);
         for (int i = 0; i < size; ++i) {
-            int str_size = offsets[i] - offsets[i - 1] - 1;
+            int str_size = offsets[i] - offsets[i - 1];
             res[i] = str_size;
         }
         return Status::OK();
@@ -94,8 +92,8 @@ struct StringUtf8LengthImpl {
         res.resize(size);
         for (int i = 0; i < size; ++i) {
             const char* raw_str = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            int str_size = offsets[i] - offsets[i - 1] - 1;
-            res[i] = get_char_len(StringValue(const_cast<char*>(raw_str), str_size), str_size);
+            int str_size = offsets[i] - offsets[i - 1];
+            res[i] = get_char_len(StringValue(raw_str, str_size), str_size);
         }
         return Status::OK();
     }
@@ -188,8 +186,8 @@ struct InStrOP {
             return;
         }
 
-        StringValue str_sv(const_cast<char*>(strl.data()), strl.length());
-        StringValue substr_sv(const_cast<char*>(strr.data()), strr.length());
+        StringValue str_sv(strl.data(), strl.length());
+        StringValue substr_sv(strr.data(), strr.length());
         StringSearch search(&substr_sv);
         // Hive returns positions starting from 1.
         int loc = search.search(&str_sv);
@@ -224,10 +222,10 @@ struct StringFunctionImpl {
         res.resize(size);
         for (int i = 0; i < size; ++i) {
             const char* l_raw_str = reinterpret_cast<const char*>(&ldata[loffsets[i - 1]]);
-            int l_str_size = loffsets[i] - loffsets[i - 1] - 1;
+            int l_str_size = loffsets[i] - loffsets[i - 1];
 
             const char* r_raw_str = reinterpret_cast<const char*>(&rdata[roffsets[i - 1]]);
-            int r_str_size = roffsets[i] - roffsets[i - 1] - 1;
+            int r_str_size = roffsets[i] - roffsets[i - 1];
 
             std::string_view lview(l_raw_str, l_str_size);
             std::string_view rview(r_raw_str, r_str_size);
@@ -239,28 +237,6 @@ struct StringFunctionImpl {
     }
 };
 
-struct NameReverse {
-    static constexpr auto name = "reverse";
-};
-
-struct ReverseImpl {
-    static Status vector(const ColumnString::Chars& data, const ColumnString::Offsets& offsets,
-                         ColumnString::Chars& res_data, ColumnString::Offsets& res_offsets) {
-        auto rows_count = offsets.size();
-        res_offsets.resize(rows_count);
-        res_data.reserve(data.size());
-        for (int i = 0; i < rows_count; ++i) {
-            auto src_str = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            int64_t src_len = offsets[i] - offsets[i - 1] - 1;
-            char dst[src_len];
-            simd::VStringFunctions::reverse(StringVal((uint8_t*)src_str, src_len),
-                                            StringVal((uint8_t*)dst, src_len));
-            StringOP::push_value_string(std::string_view(dst, src_len), i, res_data, res_offsets);
-        }
-        return Status::OK();
-    }
-};
-
 struct NameToLower {
     static constexpr auto name = "lower";
 };
@@ -269,21 +245,62 @@ struct NameToUpper {
     static constexpr auto name = "upper";
 };
 
-using char_transter_op = int (*)(int);
-template <char_transter_op op>
+template <typename OpName>
 struct TransferImpl {
     static Status vector(const ColumnString::Chars& data, const ColumnString::Offsets& offsets,
                          ColumnString::Chars& res_data, ColumnString::Offsets& res_offsets) {
         size_t offset_size = offsets.size();
-        res_offsets.resize(offsets.size());
-        for (size_t i = 0; i < offset_size; ++i) {
-            res_offsets[i] = offsets[i];
+        if (UNLIKELY(!offset_size)) {
+            return Status::OK();
         }
 
+        res_offsets.resize(offset_size);
+        memcpy(res_offsets.data(), offsets.data(),
+               offset_size * sizeof(ColumnString::Offsets::value_type));
+
         size_t data_length = data.size();
         res_data.resize(data_length);
-        for (size_t i = 0; i < data_length; ++i) {
-            res_data[i] = op(data[i]);
+        if constexpr (std::is_same_v<OpName, NameToUpper>) {
+            simd::VStringFunctions::to_upper(data.data(), data_length, res_data.data());
+        } else if constexpr (std::is_same_v<OpName, NameToLower>) {
+            simd::VStringFunctions::to_lower(data.data(), data_length, res_data.data());
+        }
+        return Status::OK();
+    }
+};
+
+// Capitalize first letter
+struct NameToInitcap {
+    static constexpr auto name = "initcap";
+};
+
+struct InitcapImpl {
+    static Status vector(const ColumnString::Chars& data, const ColumnString::Offsets& offsets,
+                         ColumnString::Chars& res_data, ColumnString::Offsets& res_offsets) {
+        size_t offset_size = offsets.size();
+        res_offsets.resize(offsets.size());
+        memcpy(res_offsets.data(), offsets.data(),
+               offset_size * sizeof(ColumnString::Offsets::value_type));
+
+        size_t data_length = data.size();
+        res_data.resize(data_length);
+        simd::VStringFunctions::to_lower(data.data(), data_length, res_data.data());
+
+        bool need_capitalize = true;
+        for (size_t offset_index = 0, start_index = 0; offset_index < offset_size; ++offset_index) {
+            auto end_index = res_offsets[offset_index];
+            need_capitalize = true;
+
+            for (size_t i = start_index; i < end_index; ++i) {
+                if (!::isalnum(res_data[i])) {
+                    need_capitalize = true;
+                } else if (need_capitalize) {
+                    res_data[i] = ::toupper(res_data[i]);
+                    need_capitalize = false;
+                }
+            }
+
+            start_index = end_index;
         }
         return Status::OK();
     }
@@ -310,7 +327,8 @@ struct TrimImpl {
 
         for (size_t i = 0; i < offset_size; ++i) {
             const char* raw_str = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            StringVal str(raw_str);
+            ColumnString::Offset size = offsets[i] - offsets[i - 1];
+            StringVal str(raw_str, size);
             if constexpr (is_ltrim) {
                 str = simd::VStringFunctions::ltrim(str);
             }
@@ -389,9 +407,9 @@ struct UnHexImpl {
             }
 
             auto source = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            size_t srclen = offsets[i] - offsets[i - 1] - 1;
+            size_t srclen = offsets[i] - offsets[i - 1];
 
-            if (*source == '\0' && srclen == 0) {
+            if (srclen == 0) {
                 StringOP::push_empty_string(i, dst_data, dst_offsets);
                 continue;
             }
@@ -460,9 +478,9 @@ struct ToBase64Impl {
             }
 
             auto source = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            size_t srclen = offsets[i] - offsets[i - 1] - 1;
+            size_t srclen = offsets[i] - offsets[i - 1];
 
-            if (*source == '\0' && srclen == 0) {
+            if (srclen == 0) {
                 StringOP::push_null_string(i, dst_data, dst_offsets, null_map);
                 continue;
             }
@@ -500,9 +518,9 @@ struct FromBase64Impl {
             }
 
             auto source = reinterpret_cast<const char*>(&data[offsets[i - 1]]);
-            size_t srclen = offsets[i] - offsets[i - 1] - 1;
+            size_t srclen = offsets[i] - offsets[i - 1];
 
-            if (*source == '\0' && srclen == 0) {
+            if (srclen == 0) {
                 StringOP::push_null_string(i, dst_data, dst_offsets, null_map);
                 continue;
             }
@@ -540,10 +558,10 @@ struct StringAppendTrailingCharIfAbsent {
         for (size_t i = 0; i < input_rows_count; ++i) {
             buffer.clear();
 
-            int l_size = loffsets[i] - loffsets[i - 1] - 1;
+            int l_size = loffsets[i] - loffsets[i - 1];
             const auto l_raw = reinterpret_cast<const char*>(&ldata[loffsets[i - 1]]);
 
-            int r_size = roffsets[i] - roffsets[i - 1] - 1;
+            int r_size = roffsets[i] - roffsets[i - 1];
             const auto r_raw = reinterpret_cast<const char*>(&rdata[roffsets[i - 1]]);
 
             if (r_size != 1) {
@@ -605,13 +623,13 @@ using FunctionStringLocate =
 using FunctionStringFindInSet =
         FunctionBinaryToType<DataTypeString, DataTypeString, StringFindInSetImpl, NameFindInSet>;
 
-using FunctionReverse = FunctionStringToString<ReverseImpl, NameReverse>;
-
 using FunctionUnHex = FunctionStringOperateToNullType<UnHexImpl>;
 
-using FunctionToLower = FunctionStringToString<TransferImpl<::tolower>, NameToLower>;
+using FunctionToLower = FunctionStringToString<TransferImpl<NameToLower>, NameToLower>;
 
-using FunctionToUpper = FunctionStringToString<TransferImpl<::toupper>, NameToUpper>;
+using FunctionToUpper = FunctionStringToString<TransferImpl<NameToUpper>, NameToUpper>;
+
+using FunctionToInitcap = FunctionStringToString<InitcapImpl, NameToInitcap>;
 
 using FunctionLTrim = FunctionStringToString<TrimImpl<true, false>, NameLTrim>;
 
@@ -640,19 +658,23 @@ void register_function_string(SimpleFunctionFactory& factory) {
     factory.register_function<FunctionStringFindInSet>();
     factory.register_function<FunctionStringLocate>();
     factory.register_function<FunctionStringLocatePos>();
-    factory.register_function<FunctionReverse>();
+    factory.register_function<FunctionReverseCommon>();
     factory.register_function<FunctionUnHex>();
     factory.register_function<FunctionToLower>();
     factory.register_function<FunctionToUpper>();
+    factory.register_function<FunctionToInitcap>();
     factory.register_function<FunctionLTrim>();
     factory.register_function<FunctionRTrim>();
     factory.register_function<FunctionTrim>();
+    factory.register_function<FunctionConvertTo>();
     factory.register_function<FunctionSubstring<Substr3Impl>>();
     factory.register_function<FunctionSubstring<Substr2Impl>>();
     factory.register_function<FunctionLeft>();
     factory.register_function<FunctionRight>();
     factory.register_function<FunctionNullOrEmpty>();
+    factory.register_function<FunctionNotNullOrEmpty>();
     factory.register_function<FunctionStringConcat>();
+    factory.register_function<FunctionStringElt>();
     factory.register_function<FunctionStringConcatWs>();
     factory.register_function<FunctionStringAppendTrailingCharIfAbsent>();
     factory.register_function<FunctionStringRepeat>();
@@ -661,7 +683,10 @@ void register_function_string(SimpleFunctionFactory& factory) {
     factory.register_function<FunctionToBase64>();
     factory.register_function<FunctionFromBase64>();
     factory.register_function<FunctionSplitPart>();
+    factory.register_function<FunctionSplitByString>();
     factory.register_function<FunctionStringMd5AndSM3<MD5Sum>>();
+    factory.register_function<FunctionSubstringIndex>();
+    factory.register_function<FunctionExtractURLParameter>();
     factory.register_function<FunctionStringParseUrl>();
     factory.register_function<FunctionMoneyFormat<MoneyFormatDoubleImpl>>();
     factory.register_function<FunctionMoneyFormat<MoneyFormatInt64Impl>>();
@@ -669,6 +694,11 @@ void register_function_string(SimpleFunctionFactory& factory) {
     factory.register_function<FunctionMoneyFormat<MoneyFormatDecimalImpl>>();
     factory.register_function<FunctionStringMd5AndSM3<SM3Sum>>();
     factory.register_function<FunctionReplace>();
+    factory.register_function<FunctionMask>();
+    factory.register_function<FunctionMaskPartial<true>>();
+    factory.register_function<FunctionMaskPartial<false>>();
+    factory.register_function<FunctionSubReplace<SubReplaceThreeImpl>>();
+    factory.register_function<FunctionSubReplace<SubReplaceFourImpl>>();
 
     factory.register_alias(FunctionLeft::name, "strleft");
     factory.register_alias(FunctionRight::name, "strright");
