@@ -17,14 +17,14 @@
 
 #include "vec/core/types.h"
 #include "vec/data_types/number_traits.h"
+#include "vec/functions/function_binary_arithmetic.h"
 #include "vec/functions/function_const.h"
-#include "vec/functions/function_binary_arithmetic.h"
-#include "vec/functions/function_binary_arithmetic.h"
 #include "vec/functions/function_math_unary.h"
 #include "vec/functions/function_math_unary_to_null_type.h"
 #include "vec/functions/function_string.h"
 #include "vec/functions/function_totype.h"
 #include "vec/functions/function_unary_arithmetic.h"
+#include "vec/functions/round.h"
 #include "vec/functions/simple_function_factory.h"
 
 namespace doris::vectorized {
@@ -127,18 +127,18 @@ struct ExpName {
 };
 using FunctionExp = FunctionMathUnary<UnaryFunctionVectorized<ExpName, std::exp>>;
 
-#define LOG_FUNCTION_IMPL(CLASS, NAME, FUNC)                                    \
-struct CLASS##Impl {                                                            \
-    using Type = DataTypeFloat64;                                               \
-    using RetType = Float64;                                                    \
-    static constexpr auto name = #NAME;                                         \
-    template <typename T, typename U>                                           \
-    static void execute(const T* src, U* dst, UInt8& null_flag) {               \
-        null_flag = src[0] <= 0;                                                \
-        dst[0] = static_cast<U>(FUNC((double)src[0]));                          \
-    }                                                                           \
-};                                                                              \
-using Function##CLASS = FunctionMathUnaryToNullType<CLASS##Impl>;
+#define LOG_FUNCTION_IMPL(CLASS, NAME, FUNC)                          \
+    struct CLASS##Impl {                                              \
+        using Type = DataTypeFloat64;                                 \
+        using RetType = Float64;                                      \
+        static constexpr auto name = #NAME;                           \
+        template <typename T, typename U>                             \
+        static void execute(const T* src, U* dst, UInt8& null_flag) { \
+            null_flag = src[0] <= 0;                                  \
+            dst[0] = static_cast<U>(FUNC((double)src[0]));            \
+        }                                                             \
+    };                                                                \
+    using Function##CLASS = FunctionMathUnaryToNullType<CLASS##Impl>;
 
 LOG_FUNCTION_IMPL(Log10, log10, std::log10);
 LOG_FUNCTION_IMPL(Log2, log2, std::log2);
@@ -180,16 +180,11 @@ struct LogImpl {
     static inline Result apply(A a, B b, UInt8& is_null) {
         is_null = a <= 0 || b <= 0 || std::fabs(a - 1.0) < EPSILON;
         return static_cast<Float64>(std::log(static_cast<Float64>(b)) /
-                                      std::log(static_cast<Float64>(a)));
+                                    std::log(static_cast<Float64>(a)));
     }
 };
 using FunctionLog = FunctionBinaryArithmetic<LogImpl, LogName, true>;
 
-struct CeilName {
-    static constexpr auto name = "ceil";
-};
-using FunctionCeil = FunctionMathUnary<UnaryFunctionVectorized<CeilName, std::ceil, DataTypeInt64>>;
-
 template <typename A>
 struct SignImpl {
     using ResultType = Int8;
@@ -235,9 +230,7 @@ template <typename A>
 struct NegativeImpl {
     using ResultType = A;
 
-    static inline ResultType apply(A a) {
-        return -a;
-    }
+    static inline ResultType apply(A a) { return -a; }
 };
 
 struct NameNegative {
@@ -269,16 +262,16 @@ struct SqrtName {
 };
 using FunctionSqrt = FunctionMathUnary<UnaryFunctionVectorized<SqrtName, std::sqrt>>;
 
+struct CbrtName {
+    static constexpr auto name = "cbrt";
+};
+using FunctionCbrt = FunctionMathUnary<UnaryFunctionVectorized<CbrtName, std::cbrt>>;
+
 struct TanName {
     static constexpr auto name = "tan";
 };
 using FunctionTan = FunctionMathUnary<UnaryFunctionVectorized<TanName, std::tan>>;
 
-struct FloorName {
-    static constexpr auto name = "floor";
-};
-using FunctionFloor = FunctionMathUnary<UnaryFunctionVectorized<FloorName, std::floor, DataTypeInt64>>;
-
 template <typename A>
 struct RadiansImpl {
     using ResultType = A;
@@ -317,7 +310,7 @@ struct BinImpl {
     static constexpr auto TYPE_INDEX = TypeIndex::Int64;
     using Type = Int64;
     using ReturnColumnType = ColumnString;
-    
+
     static std::string bin_impl(Int64 value) {
         uint64_t n = static_cast<uint64_t>(value);
         const size_t max_bits = sizeof(uint64_t) * 8;
@@ -328,13 +321,12 @@ struct BinImpl {
         } while (n >>= 1);
         return std::string(result + index, max_bits - index);
     }
-    
+
     static Status vector(const ColumnInt64::Container& data, ColumnString::Chars& res_data,
                          ColumnString::Offsets& res_offsets) {
-        
         res_offsets.resize(data.size());
         size_t input_size = res_offsets.size();
-        
+
         for (size_t i = 0; i < input_size; ++i) {
             StringOP::push_value_string(bin_impl(data[i]), i, res_data, res_offsets);
         }
@@ -344,30 +336,6 @@ struct BinImpl {
 
 using FunctionBin = FunctionUnaryToType<BinImpl, NameBin>;
 
-struct RoundName {
-    static constexpr auto name = "round";
-};
-
-/// round(double)-->int64
-/// key_str:roundFloat64
-template <typename Name>
-struct RoundOneImpl {
-    using Type = DataTypeInt64;
-    static constexpr auto name = RoundName::name;
-    static constexpr auto rows_per_iteration = 1;
-    static constexpr bool always_returns_float64 = false;
-    
-    static DataTypes get_variadic_argument_types() {
-        return {std::make_shared<vectorized::DataTypeFloat64>()};
-    }
-    
-    template <typename T, typename U>
-    static void execute(const T* src, U* dst) {
-        dst[0] = static_cast<Int64>(std::round(static_cast<Float64>(src[0])));
-    }
-};
-using FunctionRoundOne = FunctionMathUnary<RoundOneImpl<RoundName>>;
-
 template <typename A, typename B>
 struct PowImpl {
     using ResultType = double;
@@ -376,7 +344,7 @@ struct PowImpl {
     template <typename type>
     static inline double apply(A a, B b) {
         /// Next everywhere, static_cast - so that there is no wrong result in expressions of the form Int64 c = UInt32(a) * Int32(-1).
-        return std::pow((double)a, (double) b);
+        return std::pow((double)a, (double)b);
     }
 };
 struct PowName {
@@ -384,52 +352,84 @@ struct PowName {
 };
 using FunctionPow = FunctionBinaryArithmetic<PowImpl, PowName, false>;
 
-template <typename A, typename B>
-struct TruncateImpl {
-    using ResultType = double;
-    static const constexpr bool allow_decimal = false;
-
-    template <typename type>
-    static inline double apply(A a, B b) {
-        /// Next everywhere, static_cast - so that there is no wrong result in expressions of the form Int64 c = UInt32(a) * Int32(-1).
-        return static_cast<Float64>(my_double_round(
-                static_cast<Float64>(a), static_cast<Int32>(b), false, true));
-    }
-};
 struct TruncateName {
     static constexpr auto name = "truncate";
 };
-using FunctionTruncate = FunctionBinaryArithmetic<TruncateImpl, TruncateName, false>;
+
+struct CeilName {
+    static constexpr auto name = "ceil";
+};
+
+struct FloorName {
+    static constexpr auto name = "floor";
+};
+
+struct RoundName {
+    static constexpr auto name = "round";
+};
 
 /// round(double,int32)-->double
 /// key_str:roundFloat64Int32
-template <typename A, typename B>
-struct RoundTwoImpl {
-    using ResultType = double;
-    static const constexpr bool allow_decimal = false;
+template <typename Name>
+struct DoubleRoundTwoImpl {
+    static constexpr auto name = Name::name;
 
     static DataTypes get_variadic_argument_types() {
         return {std::make_shared<vectorized::DataTypeFloat64>(),
                 std::make_shared<vectorized::DataTypeInt32>()};
     }
+};
 
-    template <typename type>
-    static inline double apply(A a, B b) {
-        /// Next everywhere, static_cast - so that there is no wrong result in expressions of the form Int64 c = UInt32(a) * Int32(-1).
-        return static_cast<Float64>(my_double_round(
-                static_cast<Float64>(a), static_cast<Int32>(b), false, false));
+template <typename Name>
+struct DoubleRoundOneImpl {
+    static constexpr auto name = Name::name;
+
+    static DataTypes get_variadic_argument_types() {
+        return {std::make_shared<vectorized::DataTypeFloat64>()};
+    }
+};
+
+template <typename Name>
+struct DecimalRoundTwoImpl {
+    static constexpr auto name = Name::name;
+
+    static DataTypes get_variadic_argument_types() {
+        return {std::make_shared<vectorized::DataTypeDecimal<Decimal32>>(9, 0),
+                std::make_shared<vectorized::DataTypeInt32>()};
+    }
+};
+
+template <typename Name>
+struct DecimalRoundOneImpl {
+    static constexpr auto name = Name::name;
+
+    static DataTypes get_variadic_argument_types() {
+        return {std::make_shared<vectorized::DataTypeDecimal<Decimal32>>(9, 0)};
     }
 };
-using FunctionRoundTwo = FunctionBinaryArithmetic<RoundTwoImpl, RoundName, false>;
 
 // TODO: Now math may cause one thread compile time too long, because the function in math
 // so mush. Split it to speed up compile time in the future
 void register_function_math(SimpleFunctionFactory& factory) {
+#define REGISTER_ROUND_FUNCTIONS(IMPL)                                                        \
+    factory.register_function<                                                                \
+            FunctionRounding<IMPL<RoundName>, RoundingMode::Round, TieBreakingMode::Auto>>(); \
+    factory.register_function<                                                                \
+            FunctionRounding<IMPL<FloorName>, RoundingMode::Floor, TieBreakingMode::Auto>>(); \
+    factory.register_function<                                                                \
+            FunctionRounding<IMPL<CeilName>, RoundingMode::Ceil, TieBreakingMode::Auto>>();   \
+    factory.register_function<                                                                \
+            FunctionRounding<IMPL<TruncateName>, RoundingMode::Trunc, TieBreakingMode::Auto>>();
+
+    REGISTER_ROUND_FUNCTIONS(DecimalRoundOneImpl)
+    REGISTER_ROUND_FUNCTIONS(DecimalRoundTwoImpl)
+    REGISTER_ROUND_FUNCTIONS(DoubleRoundOneImpl)
+    REGISTER_ROUND_FUNCTIONS(DoubleRoundTwoImpl)
+    factory.register_alias("round", "dround");
     factory.register_function<FunctionAcos>();
     factory.register_function<FunctionAsin>();
     factory.register_function<FunctionAtan>();
     factory.register_function<FunctionCos>();
-    factory.register_function<FunctionCeil>();
     factory.register_alias("ceil", "dceil");
     factory.register_alias("ceil", "ceiling");
     factory.register_function<FunctionE>();
@@ -447,20 +447,17 @@ void register_function_math(SimpleFunctionFactory& factory) {
     factory.register_function<FunctionSin>();
     factory.register_function<FunctionSqrt>();
     factory.register_alias("sqrt", "dsqrt");
+    factory.register_function<FunctionCbrt>();
     factory.register_function<FunctionTan>();
-    factory.register_function<FunctionFloor>();
     factory.register_alias("floor", "dfloor");
-    factory.register_function<FunctionRoundOne>();
-    factory.register_function<FunctionRoundTwo>();
     factory.register_function<FunctionPow>();
     factory.register_alias("pow", "power");
     factory.register_alias("pow", "dpow");
     factory.register_alias("pow", "fpow");
     factory.register_function<FunctionExp>();
     factory.register_alias("exp", "dexp");
-    factory.register_function<FunctionTruncate>();
     factory.register_function<FunctionRadians>();
     factory.register_function<FunctionDegrees>();
     factory.register_function<FunctionBin>();
 }
-} // namespace doris::vectorized
\ No newline at end of file
+} // namespace doris::vectorized
