@@ -20,7 +20,6 @@
 #include <gtest/gtest.h>
 
 #include "runtime/exec_env.h"
-#include "runtime/runtime_state.h"
 #include "runtime/test_env.h"
 #include "testutil/function_utils.h"
 #include "udf/udf.h"
@@ -40,23 +39,13 @@ class TimestampFunctionsTest : public testing::Test {
 public:
     TimestampFunctionsTest() {}
 
-    void SetUp() {
-        TQueryGlobals globals;
-        globals.__set_now_string("2019-08-06 01:38:57");
-        globals.__set_timestamp_ms(1565080737805);
-        globals.__set_time_zone("America/Los_Angeles");
-        state = new RuntimeState(globals);
-        utils = new FunctionUtils(state);
+    void SetUp() override {
+        utils = new FunctionUtils();
         ctx = utils->get_fn_ctx();
     }
 
-    void TearDown() {
-        delete state;
-        delete utils;
-    }
+    void TearDown() override { delete utils; }
 
-private:
-    RuntimeState* state = nullptr;
     FunctionUtils* utils = nullptr;
     FunctionContext* ctx = nullptr;
 };
@@ -123,16 +112,14 @@ TEST_F(TimestampFunctionsTest, day_of_month_test) {
     dtv1.set_type(TIME_DATETIME);
     doris_udf::DateTimeVal tv1;
     dtv1.to_datetime_val(&tv1);
-    EXPECT_EQ(false, TimestampFunctions::day_of_month(context, tv1).is_null);
-    EXPECT_EQ(1, TimestampFunctions::day_of_month(context, tv1).val);
+    EXPECT_EQ(true, TimestampFunctions::day_of_month(context, tv1).is_null);
 
     // 2020-01-00 00:00:00
     DateTimeValue dtv2(20200100000000);
     dtv2.set_type(TIME_DATETIME);
     doris_udf::DateTimeVal tv2;
     dtv2.to_datetime_val(&tv2);
-    EXPECT_EQ(false, TimestampFunctions::day_of_month(context, tv2).is_null);
-    EXPECT_EQ(0, TimestampFunctions::day_of_month(context, tv2).val);
+    EXPECT_EQ(true, TimestampFunctions::day_of_month(context, tv2).is_null);
 
     // 2020-02-29 00:00:00
     DateTimeValue dtv3(20200229000000);
@@ -343,7 +330,7 @@ TEST_F(TimestampFunctionsTest, now) {
 }
 
 TEST_F(TimestampFunctionsTest, from_unix) {
-    IntVal unixtimestamp(1565080737);
+    IntVal unixtimestamp(1565026737);
     StringVal sval = TimestampFunctions::from_unix(ctx, unixtimestamp);
     EXPECT_EQ("2019-08-06 01:38:57", std::string((char*)sval.ptr, sval.len));
 
@@ -356,9 +343,9 @@ TEST_F(TimestampFunctionsTest, to_unix) {
     DateTimeVal dt_val;
     dt_val.packed_time = 1847544683002068992;
     dt_val.type = TIME_DATETIME;
-    EXPECT_EQ(1565080737, TimestampFunctions::to_unix(ctx).val);
-    EXPECT_EQ(1565080737, TimestampFunctions::to_unix(ctx, dt_val).val);
-    EXPECT_EQ(1565080737, TimestampFunctions::to_unix(ctx, StringVal("2019-08-06 01:38:57"),
+    EXPECT_EQ(1565026737, TimestampFunctions::to_unix(ctx).val);
+    EXPECT_EQ(1565026737, TimestampFunctions::to_unix(ctx, dt_val).val);
+    EXPECT_EQ(1565026737, TimestampFunctions::to_unix(ctx, StringVal("2019-08-06 01:38:57"),
                                                       "%Y-%m-%d %H:%i:%S")
                                   .val);
 
