@@ -21,6 +21,8 @@
 
 namespace doris {
 
+static int kNumShards = StoragePageCache::kDefaultNumShards;
+
 class StoragePageCacheTest : public testing::Test {
 public:
     StoragePageCacheTest() {}
@@ -29,7 +31,7 @@ public:
 
 // All cache space is allocated to data pages
 TEST(StoragePageCacheTest, data_page_only) {
-    StoragePageCache cache(kNumShards * 2048, 0);
+    StoragePageCache cache(kNumShards * 2048, 0, kNumShards);
 
     StoragePageCache::CacheKey key("abc", 0);
     StoragePageCache::CacheKey memory_key("mem", 0);
@@ -89,7 +91,7 @@ TEST(StoragePageCacheTest, data_page_only) {
 
 // All cache space is allocated to index pages
 TEST(StoragePageCacheTest, index_page_only) {
-    StoragePageCache cache(kNumShards * 2048, 100);
+    StoragePageCache cache(kNumShards * 2048, 100, kNumShards);
 
     StoragePageCache::CacheKey key("abc", 0);
     StoragePageCache::CacheKey memory_key("mem", 0);
@@ -149,7 +151,7 @@ TEST(StoragePageCacheTest, index_page_only) {
 
 // Cache space is allocated by index_page_cache_ratio
 TEST(StoragePageCacheTest, mixed_pages) {
-    StoragePageCache cache(kNumShards * 2048, 10);
+    StoragePageCache cache(kNumShards * 2048, 10, kNumShards);
 
     StoragePageCache::CacheKey data_key("data", 0);
     StoragePageCache::CacheKey index_key("index", 0);
