@@ -45,7 +45,6 @@
 #include "util/barrier.h"
 #include "util/countdown_latch.h"
 #include "util/metrics.h"
-#include "util/monotime.h"
 #include "util/random.h"
 #include "util/scoped_cleanup.h"
 #include "util/spinlock.h"
@@ -62,6 +61,7 @@ using strings::Substitute;
 DECLARE_int32(thread_inject_start_latency_ms);
 
 namespace doris {
+using namespace ErrorCode;
 
 static const char* kDefaultPoolName = "test";
 
@@ -155,7 +155,7 @@ TEST_F(ThreadPoolTest, TestThreadPoolWithNoMinimum) {
     EXPECT_TRUE(rebuild_pool_with_builder(ThreadPoolBuilder(kDefaultPoolName)
                                                   .set_min_threads(0)
                                                   .set_max_threads(3)
-                                                  .set_idle_timeout(MonoDelta::FromMilliseconds(1)))
+                                                  .set_idle_timeout(std::chrono::milliseconds(1)))
                         .ok());
 
     // There are no threads to start with.
@@ -228,7 +228,7 @@ TEST_F(ThreadPoolTest, TestRace) {
     EXPECT_TRUE(rebuild_pool_with_builder(ThreadPoolBuilder(kDefaultPoolName)
                                                   .set_min_threads(0)
                                                   .set_max_threads(1)
-                                                  .set_idle_timeout(MonoDelta::FromMicroseconds(1)))
+                                                  .set_idle_timeout(std::chrono::microseconds(1)))
                         .ok());
 
     for (int i = 0; i < 500; i++) {
@@ -242,7 +242,7 @@ TEST_F(ThreadPoolTest, TestRace) {
         l.wait();
         // Sleeping a different amount in each iteration makes it more likely to hit
         // the bug.
-        SleepFor(MonoDelta::FromMicroseconds(i));
+        std::this_thread::sleep_for(std::chrono::microseconds(i));
     }
 }
 
@@ -250,7 +250,7 @@ TEST_F(ThreadPoolTest, TestVariableSizeThreadPool) {
     EXPECT_TRUE(rebuild_pool_with_builder(ThreadPoolBuilder(kDefaultPoolName)
                                                   .set_min_threads(1)
                                                   .set_max_threads(4)
-                                                  .set_idle_timeout(MonoDelta::FromMilliseconds(1)))
+                                                  .set_idle_timeout(std::chrono::milliseconds(1)))
                         .ok());
 
     // There is 1 thread to start with.
@@ -289,7 +289,7 @@ TEST_F(ThreadPoolTest, TestMaxQueueSize) {
     EXPECT_TRUE(_pool->submit(SlowTask::new_slow_task(&latch)).ok());
     EXPECT_TRUE(_pool->submit(SlowTask::new_slow_task(&latch)).ok());
     Status s = _pool->submit(SlowTask::new_slow_task(&latch));
-    CHECK(s.is_service_unavailable()) << "Expected failure due to queue blowout:" << s.to_string();
+    CHECK(s.is<SERVICE_UNAVAILABLE>()) << "Expected failure due to queue blowout:" << s.to_string();
     latch.count_down();
     _pool->wait();
     _pool->shutdown();
@@ -309,7 +309,7 @@ TEST_F(ThreadPoolTest, TestZeroQueueSize) {
         EXPECT_TRUE(_pool->submit(SlowTask::new_slow_task(&latch)).ok());
     }
     Status s = _pool->submit(SlowTask::new_slow_task(&latch));
-    EXPECT_TRUE(s.is_service_unavailable()) << s.to_string();
+    EXPECT_TRUE(s.is<SERVICE_UNAVAILABLE>()) << s.to_string();
     latch.count_down();
     _pool->wait();
     _pool->shutdown();
@@ -328,9 +328,15 @@ TEST_F(ThreadPoolTest, TestZeroQueueSize) {
 #ifndef THREAD_SANITIZER
 TEST_F(ThreadPoolTest, TestDeadlocks) {
     ::testing::FLAGS_gtest_death_test_style = "threadsafe";
+#ifdef __APPLE__
+    const char* death_msg =
+            "_ZNSt3__1L8__invokeIRNS_6__bindIMN5doris10ThreadPoolEFvvEJPS3_EEEJEEEDTclscT_fp_"
+            "spscT0_fp0_EEOS9_DpOSA_";
+#else
     const char* death_msg =
             "_ZNSt5_BindIFMN5doris10ThreadPoolEFvvEPS1_EE6__callIvJEJLm0EEEET_OSt5tupleIJDpT0_"
             "EESt12_Index_tupleIJXspT1_EEE";
+#endif
     EXPECT_DEATH(
             {
                 EXPECT_TRUE(rebuild_pool_with_min_max(1, 1).ok());
@@ -354,20 +360,25 @@ class SlowDestructorRunnable : public Runnable {
 public:
     void run() override {}
 
-    virtual ~SlowDestructorRunnable() { SleepFor(MonoDelta::FromMilliseconds(100)); }
+    virtual ~SlowDestructorRunnable() {
+        std::this_thread::sleep_for(std::chrono::milliseconds(100));
+    }
 };
 
 // Test that if a tasks's destructor is slow, it doesn't cause serialization of the tasks
 // in the queue.
 TEST_F(ThreadPoolTest, TestSlowDestructor) {
     EXPECT_TRUE(rebuild_pool_with_min_max(1, 20).ok());
-    MonoTime start = MonoTime::Now();
+    auto start = std::chrono::system_clock::now();
     for (int i = 0; i < 100; i++) {
         shared_ptr<Runnable> task(new SlowDestructorRunnable());
         EXPECT_TRUE(_pool->submit(std::move(task)).ok());
     }
     _pool->wait();
-    EXPECT_LT((MonoTime::Now() - start).ToSeconds(), 5);
+    EXPECT_LT(std::chrono::duration_cast<std::chrono::seconds>(std::chrono::system_clock::now() -
+                                                               start)
+                      .count(),
+              5);
 }
 
 // For test cases that should run with both kinds of tokens.
@@ -382,7 +393,7 @@ TEST_P(ThreadPoolTestTokenTypes, TestTokenSubmitAndWait) {
     std::unique_ptr<ThreadPoolToken> t = _pool->new_token(GetParam());
     int i = 0;
     Status status = t->submit_func([&]() {
-        SleepFor(MonoDelta::FromMilliseconds(1));
+        std::this_thread::sleep_for(std::chrono::milliseconds(1));
         i++;
     });
     EXPECT_TRUE(status.ok());
@@ -401,7 +412,7 @@ TEST_F(ThreadPoolTest, TestTokenSubmitsProcessedSerially) {
         // appends if the submissions did execute in parallel.
         int sleep_ms = r.Next() % 5;
         Status status = t->submit_func([&result, c, sleep_ms]() {
-            SleepFor(MonoDelta::FromMilliseconds(sleep_ms));
+            std::this_thread::sleep_for(std::chrono::milliseconds(sleep_ms));
             result += c;
         });
         EXPECT_TRUE(status.ok());
@@ -485,7 +496,7 @@ TEST_P(ThreadPoolTestTokenTypes, TestTokenShutdown) {
     t1->shutdown();
 
     // We can no longer submit to t1 but we can still submit to t2.
-    EXPECT_TRUE(t1->submit_func([]() {}).is_service_unavailable());
+    EXPECT_TRUE(t1->submit_func([]() {}).is<SERVICE_UNAVAILABLE>());
     EXPECT_TRUE(t2->submit_func([]() {}).ok());
 
     // Unblock t2's tasks.
@@ -511,7 +522,7 @@ TEST_P(ThreadPoolTestTokenTypes, TestTokenWaitForAll) {
         int sleep_ms = r.Next() % 5;
 
         auto task = [&v, sleep_ms]() {
-            SleepFor(MonoDelta::FromMilliseconds(sleep_ms));
+            std::this_thread::sleep_for(std::chrono::milliseconds(sleep_ms));
             v++;
         };
 
@@ -550,7 +561,7 @@ TEST_F(ThreadPoolTest, TestFuzz) {
             int sleep_ms = r.Next() % 5;
             EXPECT_TRUE(_pool->submit_func([sleep_ms]() {
                                  // Sleep a little first to increase task overlap.
-                                 SleepFor(MonoDelta::FromMilliseconds(sleep_ms));
+                                 std::this_thread::sleep_for(std::chrono::milliseconds(sleep_ms));
                              }).ok());
         } else if (op < 75) {
             // submit with a randomly selected token.
@@ -561,9 +572,9 @@ TEST_F(ThreadPoolTest, TestFuzz) {
             int token_idx = r.Next() % tokens.size();
             Status s = tokens[token_idx]->submit_func([sleep_ms]() {
                 // Sleep a little first to increase task overlap.
-                SleepFor(MonoDelta::FromMilliseconds(sleep_ms));
+                std::this_thread::sleep_for(std::chrono::milliseconds(sleep_ms));
             });
-            EXPECT_TRUE(s.ok() || s.is_service_unavailable());
+            EXPECT_TRUE(s.ok() || s.is<SERVICE_UNAVAILABLE>());
         } else if (op < 85) {
             // Allocate a token with a randomly selected policy.
             ThreadPool::ExecutionMode mode = r.Next() % 2 ? ThreadPool::ExecutionMode::SERIAL
@@ -622,7 +633,7 @@ TEST_P(ThreadPoolTestTokenTypes, TestTokenSubmissionsAdhereToMaxQueueSize) {
     EXPECT_TRUE(t->submit(SlowTask::new_slow_task(&latch)).ok());
     EXPECT_TRUE(t->submit(SlowTask::new_slow_task(&latch)).ok());
     Status s = t->submit(SlowTask::new_slow_task(&latch));
-    EXPECT_TRUE(s.is_service_unavailable());
+    EXPECT_TRUE(s.is<SERVICE_UNAVAILABLE>());
 }
 
 TEST_F(ThreadPoolTest, TestTokenConcurrency) {
@@ -687,7 +698,7 @@ TEST_F(ThreadPoolTest, TestTokenConcurrency) {
 
                 // Sleep a bit, otherwise this thread outpaces the other threads and
                 // nothing interesting happens to most tokens.
-                SleepFor(MonoDelta::FromMicroseconds(10));
+                std::this_thread::sleep_for(std::chrono::microseconds(10));
             }
             total_num_tokens_cycled += num_tokens_cycled;
         });
@@ -729,16 +740,16 @@ TEST_F(ThreadPoolTest, TestTokenConcurrency) {
                 int sleep_ms = rng.Next() % 5;
                 Status s = GetRandomToken()->submit_func([sleep_ms]() {
                     // Sleep a little first so that tasks are running during other events.
-                    SleepFor(MonoDelta::FromMilliseconds(sleep_ms));
+                    std::this_thread::sleep_for(std::chrono::milliseconds(sleep_ms));
                 });
-                CHECK(s.ok() || s.is_service_unavailable());
+                CHECK(s.ok() || s.is<SERVICE_UNAVAILABLE>());
                 num_tokens_submitted++;
             }
             total_num_tokens_submitted += num_tokens_submitted;
         });
     }
 
-    SleepFor(MonoDelta::FromSeconds(kTestRuntimeSecs));
+    std::this_thread::sleep_for(std::chrono::seconds(kTestRuntimeSecs));
     latch.count_down();
     for (auto& t : threads) {
         t.join();
@@ -766,7 +777,7 @@ TEST_F(ThreadPoolTest, TestNormal) {
             .set_min_threads(0)
             .set_max_threads(5)
             .set_max_queue_size(10)
-            .set_idle_timeout(MonoDelta::FromMilliseconds(2000))
+            .set_idle_timeout(std::chrono::milliseconds(2000))
             .build(&thread_pool);
 
     std::unique_ptr<ThreadPoolToken> token1 =
@@ -814,7 +825,7 @@ TEST_F(ThreadPoolTest, TestThreadPoolDynamicAdjustMaximumMinimum) {
     EXPECT_TRUE(rebuild_pool_with_builder(ThreadPoolBuilder(kDefaultPoolName)
                                                   .set_min_threads(3)
                                                   .set_max_threads(3)
-                                                  .set_idle_timeout(MonoDelta::FromMilliseconds(1)))
+                                                  .set_idle_timeout(std::chrono::milliseconds(1)))
                         .ok());
 
     EXPECT_EQ(3, _pool->min_threads());
@@ -865,7 +876,7 @@ TEST_F(ThreadPoolTest, TestThreadPoolDynamicAdjustMaximumMinimum) {
     latch_1.count_down();
     latch_2.count_down();
     latch_3.count_down();
-    SleepFor(MonoDelta::FromMilliseconds(500));
+    std::this_thread::sleep_for(std::chrono::milliseconds(500));
     EXPECT_EQ(4, _pool->num_threads());
 
     EXPECT_TRUE(_pool->submit(SlowTask::new_slow_task(&latch_8)).ok());
@@ -881,7 +892,7 @@ TEST_F(ThreadPoolTest, TestThreadPoolDynamicAdjustMaximumMinimum) {
     latch_7.count_down();
     latch_8.count_down();
     latch_9.count_down();
-    SleepFor(MonoDelta::FromMilliseconds(500));
+    std::this_thread::sleep_for(std::chrono::milliseconds(500));
     EXPECT_EQ(2, _pool->num_threads());
 
     _pool->wait();
@@ -895,7 +906,7 @@ TEST_F(ThreadPoolTest, TestThreadTokenSerial) {
             .set_min_threads(0)
             .set_max_threads(1)
             .set_max_queue_size(10)
-            .set_idle_timeout(MonoDelta::FromMilliseconds(2000))
+            .set_idle_timeout(std::chrono::milliseconds(2000))
             .build(&thread_pool);
 
     std::unique_ptr<ThreadPoolToken> token1 =
@@ -903,13 +914,13 @@ TEST_F(ThreadPoolTest, TestThreadTokenSerial) {
     token1->submit_func(std::bind(&MyFunc, 0, 1));
     std::cout << "after submit 1" << std::endl;
     token1->wait();
-    EXPECT_EQ(0, token1->num_tasks());
+    ASSERT_EQ(0, token1->num_tasks());
     for (int i = 0; i < 10; i++) {
         token1->submit_func(std::bind(&MyFunc, i, 1));
     }
     std::cout << "after submit 1" << std::endl;
     token1->wait();
-    EXPECT_EQ(0, token1->num_tasks());
+    ASSERT_EQ(0, token1->num_tasks());
 }
 
 TEST_F(ThreadPoolTest, TestThreadTokenConcurrent) {
@@ -918,7 +929,7 @@ TEST_F(ThreadPoolTest, TestThreadTokenConcurrent) {
             .set_min_threads(0)
             .set_max_threads(1)
             .set_max_queue_size(10)
-            .set_idle_timeout(MonoDelta::FromMilliseconds(2000))
+            .set_idle_timeout(std::chrono::milliseconds(2000))
             .build(&thread_pool);
 
     std::unique_ptr<ThreadPoolToken> token1 =
@@ -928,7 +939,7 @@ TEST_F(ThreadPoolTest, TestThreadTokenConcurrent) {
     }
     std::cout << "after submit 1" << std::endl;
     token1->wait();
-    EXPECT_EQ(0, token1->num_tasks());
+    ASSERT_EQ(0, token1->num_tasks());
 }
 
 } // namespace doris
