@@ -14,6 +14,9 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+// This file is copied from
+// https://github.com/apache/impala/blob/branch-2.9.0/fe/src/main/java/org/apache/impala/InlineViewRef.java
+// and modified by Doris
 
 package org.apache.doris.analysis;
 
@@ -25,11 +28,11 @@ import org.apache.doris.common.ErrorCode;
 import org.apache.doris.common.ErrorReport;
 import org.apache.doris.common.UserException;
 import org.apache.doris.rewrite.ExprRewriter;
+import org.apache.doris.thrift.TNullSide;
 
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -97,14 +100,18 @@ public class InlineViewRef extends TableRef {
     public InlineViewRef(View view, TableRef origTblRef) {
         super(origTblRef.getName(), origTblRef.getExplicitAlias());
         queryStmt = view.getQueryStmt().clone();
-        if (view.isLocalView()) queryStmt.reset();
+        if (view.isLocalView()) {
+            queryStmt.reset();
+        }
         this.view = view;
         sMap = new ExprSubstitutionMap();
         baseTblSmap = new ExprSubstitutionMap();
         setJoinAttrs(origTblRef);
         explicitColLabels = view.getColLabels();
         // Set implicit aliases if no explicit one was given.
-        if (hasExplicitAlias) return;
+        if (hasExplicitAlias()) {
+            return;
+        }
         // TODO(zc)
         // view_.getTableName().toString().toLowerCase(), view.getName().toLowerCase()
         if (view.isLocalView()) {
@@ -130,7 +137,9 @@ public class InlineViewRef extends TableRef {
         baseTblSmap = other.baseTblSmap.clone();
     }
 
-    public List<String> getExplicitColLabels() { return explicitColLabels; }
+    public List<String> getExplicitColLabels() {
+        return explicitColLabels;
+    }
 
     public List<String> getColLabels() {
         if (explicitColLabels != null) {
@@ -142,12 +151,12 @@ public class InlineViewRef extends TableRef {
 
     @Override
     public void reset() {
-      super.reset();
-      queryStmt.reset();
-      inlineViewAnalyzer = null;
-      materializedTupleIds.clear();
-      sMap.clear();
-      baseTblSmap.clear();
+        super.reset();
+        queryStmt.reset();
+        inlineViewAnalyzer = null;
+        materializedTupleIds.clear();
+        sMap.clear();
+        baseTblSmap.clear();
     }
 
     @Override
@@ -171,7 +180,7 @@ public class InlineViewRef extends TableRef {
             return;
         }
 
-        if (view == null && !hasExplicitAlias) {
+        if (view == null && !hasExplicitAlias()) {
             ErrorReport.reportAnalysisException(ErrorCode.ERR_DERIVED_MUST_HAVE_ALIAS);
         }
 
@@ -182,10 +191,10 @@ public class InlineViewRef extends TableRef {
             inlineViewAnalyzer.setExplicitViewAlias(aliases[0]);
         }
         queryStmt.analyze(inlineViewAnalyzer);
-        correlatedTupleIds_.addAll(queryStmt.getCorrelatedTupleIds(inlineViewAnalyzer));
+        correlatedTupleIds.addAll(queryStmt.getCorrelatedTupleIds(inlineViewAnalyzer));
 
         queryStmt.getMaterializedTupleIds(materializedTupleIds);
-        if (view != null && !hasExplicitAlias && !view.isLocalView()) {
+        if (view != null && !hasExplicitAlias() && !view.isLocalView()) {
             name = analyzer.getFqTableName(name);
             aliases = new String[] { name.toString(), view.getName() };
         }
@@ -212,11 +221,13 @@ public class InlineViewRef extends TableRef {
         // would alter the results of the analytic functions (see IMPALA-1243)
         // TODO: relax this a bit by allowing propagation out of the inline view (but
         // not into it)
+        List<SlotDescriptor> slots = analyzer.changeSlotToNullableOfOuterJoinedTuples();
         for (int i = 0; i < getColLabels().size(); ++i) {
             String colName = getColLabels().get(i);
             SlotDescriptor slotDesc = analyzer.registerColumnRef(getAliasAsName(), colName);
             Expr colExpr = queryStmt.getResultExprs().get(i);
             slotDesc.setSourceExpr(colExpr);
+            slotDesc.setIsNullable(slotDesc.getIsNullable() || colExpr.isNullable());
             SlotRef slotRef = new SlotRef(slotDesc);
             sMap.put(slotRef, colExpr);
             baseTblSmap.put(slotRef, queryStmt.getBaseTblResultExprs().get(i));
@@ -224,6 +235,7 @@ public class InlineViewRef extends TableRef {
                 analyzer.createAuxEquivPredicate(new SlotRef(slotDesc), colExpr.clone());
             }
         }
+        analyzer.changeSlotsToNotNullable(slots);
         if (LOG.isDebugEnabled()) {
             LOG.debug("inline view " + getUniqueAlias() + " smap: " + sMap.debugString());
             LOG.debug("inline view " + getUniqueAlias() + " baseTblSmap: " + baseTblSmap.debugString());
@@ -283,7 +295,8 @@ public class InlineViewRef extends TableRef {
                     false, null, selectItemExpr.isNullable(),
                     null, ""));
         }
-        InlineView inlineView = (view != null) ? new InlineView(view, columnList) : new InlineView(getExplicitAlias(), columnList);
+        InlineView inlineView = (view != null) ? new InlineView(view, columnList)
+                : new InlineView(getExplicitAlias(), columnList);
 
         // Create the non-materialized tuple and set the fake table in it.
         TupleDescriptor result = analyzer.getDescTbl().createTupleDescriptor();
@@ -358,7 +371,11 @@ public class InlineViewRef extends TableRef {
             if (!requiresNullWrapping(analyzer, smap.getRhs().get(i), nullSMap)) {
                 continue;
             }
-            params.add(new TupleIsNullPredicate(materializedTupleIds));
+            if (analyzer.isOuterJoinedLeftSide(materializedTupleIds.get(0))) {
+                params.add(new TupleIsNullPredicate(materializedTupleIds, TNullSide.LEFT));
+            } else {
+                params.add(new TupleIsNullPredicate(materializedTupleIds, TNullSide.RIGHT));
+            }
             params.add(NullLiteral.create(smap.getRhs().get(i).getType()));
             params.add(smap.getRhs().get(i));
             Expr ifExpr = new FunctionCallExpr("if", params);
@@ -380,19 +397,6 @@ public class InlineViewRef extends TableRef {
             return true;
         }
         return true;
-
-//        // Replace all SlotRefs in expr with NullLiterals, and wrap the result
-//        // into an IS NOT NULL predicate.
-//        Expr isNotNullLiteralPred = new IsNullPredicate(expr.clone(nullSMap), true);
-//        Preconditions.checkState(isNotNullLiteralPred.isConstant());
-//        // analyze to insert casts, etc.
-//        try {
-//            isNotNullLiteralPred.analyze(analyzer);
-//        } catch (AnalysisException e) {
-//            // this should never happen
-//            throw new InternalException("couldn't analyze predicate " + isNotNullLiteralPred.toSql(), e);
-//        }
-//        return FeSupport.EvalPredicate(isNotNullLiteralPred, analyzer.getQueryGlobals());
     }
 
     @Override
@@ -465,4 +469,24 @@ public class InlineViewRef extends TableRef {
 
         return sb.toString();
     }
+
+    @Override
+    public String tableRefToDigest() {
+        String aliasSql = null;
+        String alias = getExplicitAlias();
+        if (alias != null) {
+            aliasSql = ToSqlUtils.getIdentSql(alias);
+        }
+        if (view != null) {
+            return name.toSql() + (aliasSql == null ? "" : " " + aliasSql);
+        }
+
+        StringBuilder sb = new StringBuilder()
+                .append("(")
+                .append(queryStmt.toDigest())
+                .append(") ")
+                .append(aliasSql);
+
+        return sb.toString();
+    }
 }
