@@ -17,10 +17,10 @@
 
 package org.apache.doris.catalog;
 
-import static org.apache.doris.common.io.IOUtils.writeOptionString;
-
 import org.apache.doris.analysis.FunctionName;
 import org.apache.doris.common.AnalysisException;
+import org.apache.doris.common.UserException;
+import org.apache.doris.common.io.IOUtils;
 import org.apache.doris.common.io.Text;
 import org.apache.doris.common.io.Writable;
 import org.apache.doris.common.util.URI;
@@ -30,16 +30,16 @@ import org.apache.doris.thrift.TFunctionBinaryType;
 import com.google.common.base.Joiner;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
-
+import org.apache.commons.io.output.NullOutputStream;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
 import java.io.DataInput;
 import java.io.DataOutput;
+import java.io.DataOutputStream;
 import java.io.IOException;
 import java.util.List;
 
-
 /**
  * Base class for all functions.
  */
@@ -136,7 +136,8 @@ public class Function implements Writable {
         this(0, name, args, retType, varArgs, vectorized, NullableMode.DEPEND_ON_ARGUMENT);
     }
 
-    public Function(FunctionName name, List<Type> args, Type retType, boolean varArgs, boolean vectorized, NullableMode mode) {
+    public Function(FunctionName name, List<Type> args, Type retType,
+            boolean varArgs, boolean vectorized, NullableMode mode) {
         this(0, name, args, retType, varArgs, vectorized, mode);
     }
 
@@ -162,6 +163,26 @@ public class Function implements Writable {
         this(id, name, argTypes, retType, hasVarArgs, TFunctionBinaryType.BUILTIN, true, vectorized, mode);
     }
 
+    public Function(Function other) {
+        if (other == null) {
+            return;
+        }
+        this.id = other.id;
+        this.name = new FunctionName(other.name.getDb(), other.name.getFunction());
+        this.hasVarArgs = other.hasVarArgs;
+        this.retType = other.retType;
+        this.userVisible = other.userVisible;
+        this.nullableMode = other.nullableMode;
+        this.vectorized = other.vectorized;
+        this.binaryType = other.binaryType;
+        this.location = other.location;
+        if (other.argTypes != null) {
+            this.argTypes = new Type[other.argTypes.length];
+            System.arraycopy(other.argTypes, 0, this.argTypes, 0, other.argTypes.length);
+        }
+        this.checksum = other.checksum;
+    }
+
     public FunctionName getFunctionName() {
         return name;
     }
@@ -235,10 +256,21 @@ public class Function implements Writable {
         hasVarArgs = v;
     }
 
-    public void setId(long functionId) { this.id = functionId; }
-    public long getId() { return id; }
-    public void setChecksum(String checksum) { this.checksum = checksum; }
-    public String getChecksum() { return checksum; }
+    public void setId(long functionId) {
+        this.id = functionId;
+    }
+
+    public long getId() {
+        return id;
+    }
+
+    public void setChecksum(String checksum) {
+        this.checksum = checksum;
+    }
+
+    public String getChecksum() {
+        return checksum;
+    }
 
     // TODO(cmy): Currently we judge whether it is UDF by wheter the 'location' is set.
     // Maybe we should use a separate variable to identify,
@@ -447,7 +479,7 @@ public class Function implements Writable {
         }
     }
 
-    public TFunction toThrift() {
+    public TFunction toThrift(Type realReturnType, Type[] realArgTypes) {
         TFunction fn = new TFunction();
         fn.setSignature(signatureString());
         fn.setName(name.toThrift());
@@ -455,8 +487,20 @@ public class Function implements Writable {
         if (location != null) {
             fn.setHdfsLocation(location.getLocation());
         }
-        fn.setArgTypes(Type.toThrift(argTypes));
-        fn.setRetType(getReturnType().toThrift());
+        // `realArgTypes.length != argTypes.length` is true iff this is an aggregation function.
+        // For aggregation functions, `argTypes` here is already its real type with true precision and scale.
+        if (realArgTypes.length != argTypes.length) {
+            fn.setArgTypes(Type.toThrift(Lists.newArrayList(argTypes)));
+        } else {
+            fn.setArgTypes(Type.toThrift(Lists.newArrayList(argTypes), Lists.newArrayList(realArgTypes)));
+        }
+        // For types with different precisions and scales, return type only indicates a type with default
+        // precision and scale so we need to transform it to the correct type.
+        if (PrimitiveType.typeWithPrecision.contains(realReturnType.getPrimitiveType())) {
+            fn.setRetType(realReturnType.toThrift());
+        } else {
+            fn.setRetType(getReturnType().toThrift());
+        }
         fn.setHasVarArgs(hasVarArgs);
         // TODO: Comment field is missing?
         // fn.setComment(comment)
@@ -491,18 +535,32 @@ public class Function implements Writable {
                 return "float_val";
             case DOUBLE:
             case TIME:
+            case TIMEV2:
                 return "double_val";
             case VARCHAR:
             case CHAR:
             case HLL:
             case BITMAP:
+            case QUANTILE_STATE:
             case STRING:
                 return "string_val";
+            case JSONB:
+                return "jsonb_val";
             case DATE:
             case DATETIME:
                 return "datetime_val";
+            case DATEV2:
+                return "datev2_val";
+            case DATETIMEV2:
+                return "datetimev2_val";
             case DECIMALV2:
                 return "decimalv2_val";
+            case DECIMAL32:
+                return "decimal32_val";
+            case DECIMAL64:
+                return "decimal64_val";
+            case DECIMAL128:
+                return "decimal128_val";
             default:
                 Preconditions.checkState(false, t.toString());
                 return "";
@@ -529,18 +587,32 @@ public class Function implements Writable {
                 return "FloatVal";
             case DOUBLE:
             case TIME:
+            case TIMEV2:
                 return "DoubleVal";
             case VARCHAR:
             case CHAR:
             case HLL:
             case BITMAP:
+            case QUANTILE_STATE:
             case STRING:
                 return "StringVal";
+            case JSONB:
+                return "JsonbVal";
             case DATE:
             case DATETIME:
                 return "DateTimeVal";
+            case DATEV2:
+                return "DateV2Val";
+            case DATETIMEV2:
+                return "DateTimeV2Val";
             case DECIMALV2:
                 return "DecimalV2Val";
+            case DECIMAL32:
+                return "Decimal32Val";
+            case DECIMAL64:
+                return "Decimal64Val";
+            case DECIMAL128:
+                return "Decimal128Val";
             default:
                 Preconditions.checkState(false, t.toString());
                 return "";
@@ -600,12 +672,13 @@ public class Function implements Writable {
         FunctionType(int code) {
             this.code = code;
         }
+
         public int getCode() {
             return code;
         }
 
         public static FunctionType fromCode(int code) {
-            switch (code) {
+            switch (code) { // CHECKSTYLE IGNORE THIS LINE: missing switch default
                 case 0:
                     return ORIGIN;
                 case 1:
@@ -621,10 +694,11 @@ public class Function implements Writable {
         public void write(DataOutput output) throws IOException {
             output.writeInt(code);
         }
+
         public static FunctionType read(DataInput input) throws IOException {
             return fromCode(input.readInt());
         }
-    };
+    }
 
     protected void writeFields(DataOutput output) throws IOException {
         output.writeLong(id);
@@ -642,8 +716,8 @@ public class Function implements Writable {
         if (location != null) {
             libUrl = location.getLocation();
         }
-        writeOptionString(output, libUrl);
-        writeOptionString(output, checksum);
+        IOUtils.writeOptionString(output, libUrl);
+        IOUtils.writeOptionString(output, checksum);
     }
 
     @Override
@@ -741,7 +815,23 @@ public class Function implements Writable {
         return vectorized;
     }
 
+    public void setNullableMode(NullableMode nullableMode) {
+        this.nullableMode = nullableMode;
+    }
+
     public NullableMode getNullableMode() {
         return nullableMode;
     }
+
+    // Try to serialize this function and write to nowhere.
+    // Just for checking if we forget to implement write() method for some Exprs.
+    // To avoid FE exist when writing edit log.
+    public void checkWritable() throws UserException {
+        try {
+            DataOutputStream out = new DataOutputStream(new NullOutputStream());
+            write(out);
+        } catch (Throwable t) {
+            throw new UserException("failed to serialize function: " + functionName(), t);
+        }
+    }
 }
