@@ -22,8 +22,13 @@ import org.apache.doris.analysis.DropTableStmt;
 import org.apache.doris.analysis.RefreshDbStmt;
 import org.apache.doris.analysis.RefreshTableStmt;
 import org.apache.doris.analysis.TableName;
+import org.apache.doris.catalog.external.ExternalDatabase;
 import org.apache.doris.common.DdlException;
 import org.apache.doris.common.UserException;
+import org.apache.doris.datasource.CatalogIf;
+import org.apache.doris.datasource.ExternalCatalog;
+import org.apache.doris.datasource.ExternalObjectLog;
+import org.apache.doris.datasource.InternalCatalog;
 
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
@@ -35,39 +40,49 @@ public class RefreshManager {
     private static final Logger LOG = LogManager.getLogger(RefreshManager.class);
 
     public void handleRefreshTable(RefreshTableStmt stmt) throws UserException {
+        String catalogName = stmt.getCtl();
         String dbName = stmt.getDbName();
         String tableName = stmt.getTblName();
-        Catalog catalog = Catalog.getCurrentCatalog();
+        Env env = Env.getCurrentEnv();
 
-        // 0. check table type
-        Database db = catalog.getDbOrDdlException(dbName);
-        Table table = db.getTableNullable(tableName);
-        if (!(table instanceof IcebergTable)) {
-            throw new DdlException("Only support refresh Iceberg table.");
+        CatalogIf catalog = catalogName != null ? env.getCatalogMgr().getCatalog(catalogName) : env.getCurrentCatalog();
+
+        if (catalog == null) {
+            throw new DdlException("Catalog " + catalogName + " doesn't exist.");
         }
 
-        // 1. get iceberg properties
-        Map<String, String> icebergProperties = ((IcebergTable) table).getIcebergProperties();
-        icebergProperties.put(IcebergProperty.ICEBERG_TABLE, ((IcebergTable) table).getIcebergTbl());
-        icebergProperties.put(IcebergProperty.ICEBERG_DATABASE, ((IcebergTable) table).getIcebergDb());
-
-        // 2. drop old table
-        DropTableStmt dropTableStmt = new DropTableStmt(true, stmt.getTableName(), true);
-        catalog.dropTable(dropTableStmt);
-
-        // 3. create new table
-        CreateTableStmt createTableStmt = new CreateTableStmt(true, true,
-                stmt.getTableName(), "ICEBERG", icebergProperties, "");
-        catalog.createTable(createTableStmt);
-
+        if (catalog.getName().equals(InternalCatalog.INTERNAL_CATALOG_NAME)) {
+            // Process internal catalog iceberg external table refresh.
+            refreshInternalCtlIcebergTable(stmt, env);
+        } else {
+            // Process external catalog table refresh
+            env.getCatalogMgr().refreshExternalTable(dbName, tableName, catalogName);
+        }
         LOG.info("Successfully refresh table: {} from db: {}", tableName, dbName);
     }
 
     public void handleRefreshDb(RefreshDbStmt stmt) throws DdlException {
+        String catalogName = stmt.getCatalogName();
         String dbName = stmt.getDbName();
-        Catalog catalog = Catalog.getCurrentCatalog();
+        Env env = Env.getCurrentEnv();
+        CatalogIf catalog = catalogName != null ? env.getCatalogMgr().getCatalog(catalogName) : env.getCurrentCatalog();
 
-        Database db = catalog.getDbOrDdlException(dbName);
+        if (catalog == null) {
+            throw new DdlException("Catalog " + catalogName + " doesn't exist.");
+        }
+
+        if (catalog.getName().equals(InternalCatalog.INTERNAL_CATALOG_NAME)) {
+            // Process internal catalog iceberg external db refresh.
+            refreshInternalCtlIcebergDb(dbName, env);
+        } else {
+            // Process external catalog db refresh
+            refreshExternalCtlDb(dbName, catalog, stmt.isInvalidCache());
+        }
+        LOG.info("Successfully refresh db: {}", dbName);
+    }
+
+    private void refreshInternalCtlIcebergDb(String dbName, Env env) throws DdlException {
+        Database db = env.getInternalCatalog().getDbOrDdlException(dbName);
 
         // 0. build iceberg property
         // Since we have only persisted database properties with key-value format in DatabaseProperty,
@@ -82,14 +97,53 @@ public class RefreshManager {
         // Current database may have other types of table, which is not allowed to drop.
         for (Table table : db.getTables()) {
             if (table instanceof IcebergTable) {
-                DropTableStmt dropTableStmt = new DropTableStmt(true, new TableName(dbName, table.getName()), true);
-                catalog.dropTable(dropTableStmt);
+                DropTableStmt dropTableStmt =
+                        new DropTableStmt(true, new TableName(null, dbName, table.getName()), true);
+                env.dropTable(dropTableStmt);
             }
         }
 
         // 3. register iceberg database to recreate iceberg table
-        catalog.getIcebergTableCreationRecordMgr().registerDb(db);
+        env.getIcebergTableCreationRecordMgr().registerDb(db);
+    }
 
-        LOG.info("Successfully refresh db: {}", dbName);
+    private void refreshExternalCtlDb(String dbName, CatalogIf catalog, boolean invalidCache) throws DdlException {
+        if (!(catalog instanceof ExternalCatalog)) {
+            throw new DdlException("Only support refresh ExternalCatalog Database");
+        }
+
+        DatabaseIf db = catalog.getDbNullable(dbName);
+        if (db == null) {
+            throw new DdlException("Database " + dbName + " does not exist in catalog " + catalog.getName());
+        }
+        ((ExternalDatabase) db).setUnInitialized(invalidCache);
+        ExternalObjectLog log = new ExternalObjectLog();
+        log.setCatalogId(catalog.getId());
+        log.setDbId(db.getId());
+        log.setInvalidCache(invalidCache);
+        Env.getCurrentEnv().getEditLog().logRefreshExternalDb(log);
+    }
+
+    private void refreshInternalCtlIcebergTable(RefreshTableStmt stmt, Env env) throws UserException {
+        // 0. check table type
+        Database db = env.getInternalCatalog().getDbOrDdlException(stmt.getDbName());
+        Table table = db.getTableNullable(stmt.getTblName());
+        if (!(table instanceof IcebergTable)) {
+            throw new DdlException("Only support refresh Iceberg table.");
+        }
+
+        // 1. get iceberg properties
+        Map<String, String> icebergProperties = ((IcebergTable) table).getIcebergProperties();
+        icebergProperties.put(IcebergProperty.ICEBERG_TABLE, ((IcebergTable) table).getIcebergTbl());
+        icebergProperties.put(IcebergProperty.ICEBERG_DATABASE, ((IcebergTable) table).getIcebergDb());
+
+        // 2. drop old table
+        DropTableStmt dropTableStmt = new DropTableStmt(true, stmt.getTableName(), true);
+        env.dropTable(dropTableStmt);
+
+        // 3. create new table
+        CreateTableStmt createTableStmt = new CreateTableStmt(true, true,
+                stmt.getTableName(), "ICEBERG", icebergProperties, "");
+        env.createTable(createTableStmt);
     }
 }
