@@ -31,7 +31,6 @@ import org.apache.doris.thrift.TStorageMedium;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -182,8 +181,10 @@ public class BackendLoadStatistic {
             TStorageMedium medium = diskInfo.getStorageMedium();
             if (diskInfo.getState() == DiskState.ONLINE) {
                 // we only collect online disk's capacity
-                totalCapacityMap.put(medium, totalCapacityMap.getOrDefault(medium, 0L) + diskInfo.getTotalCapacityB());
-                totalUsedCapacityMap.put(medium, totalUsedCapacityMap.getOrDefault(medium, 0L) + (diskInfo.getTotalCapacityB() - diskInfo.getAvailableCapacityB()));
+                totalCapacityMap.put(medium, totalCapacityMap.getOrDefault(medium, 0L)
+                        + diskInfo.getTotalCapacityB());
+                totalUsedCapacityMap.put(medium, totalUsedCapacityMap.getOrDefault(medium, 0L)
+                        + (diskInfo.getTotalCapacityB() - diskInfo.getAvailableCapacityB()));
             }
 
             RootPathLoadStatistic pathStatistic = new RootPathLoadStatistic(beId, diskInfo.getRootPath(),
@@ -246,13 +247,13 @@ public class BackendLoadStatistic {
             }
         }
 
-        LOG.debug("classify path by load. storage: {} avg used percent: {}. low/mid/high: {}/{}/{}",
-                avgUsedPercent, medium, lowCounter, midCounter, highCounter);
+        LOG.debug("classify path by load. be id: {} storage: {} avg used percent: {}. low/mid/high: {}/{}/{}",
+                beId, medium, avgUsedPercent, lowCounter, midCounter, highCounter);
     }
 
     public void calcScore(Map<TStorageMedium, Double> avgClusterUsedCapacityPercentMap,
             Map<TStorageMedium, Double> avgClusterReplicaNumPerBackendMap) {
-        
+
         for (TStorageMedium medium : TStorageMedium.values()) {
             LoadScore loadScore = calcSore(totalUsedCapacityMap.getOrDefault(medium, 0L),
                     totalCapacityMap.getOrDefault(medium, 1L),
@@ -269,26 +270,28 @@ public class BackendLoadStatistic {
 
     public static LoadScore calcSore(long beUsedCapacityB, long beTotalCapacityB, long beTotalReplicaNum,
             double avgClusterUsedCapacityPercent, double avgClusterReplicaNumPerBackend) {
-        
+
         double usedCapacityPercent = (beUsedCapacityB / (double) beTotalCapacityB);
         double capacityProportion = avgClusterUsedCapacityPercent <= 0 ? 0.0
                 : usedCapacityPercent / avgClusterUsedCapacityPercent;
         double replicaNumProportion = avgClusterReplicaNumPerBackend <= 0 ? 0.0
                 : beTotalReplicaNum / avgClusterReplicaNumPerBackend;
-        
+
         LoadScore loadScore = new LoadScore();
 
         // If this backend's capacity used percent < 50%, set capacityCoefficient to 0.5.
         // Else if capacity used percent > 75%, set capacityCoefficient to 1.
         // Else, capacityCoefficient changed smoothly from 0.5 to 1 with used capacity increasing
         // Function: (2 * usedCapacityPercent - 0.5)
-        loadScore.capacityCoefficient = usedCapacityPercent < 0.5 ? 0.5
-                : (usedCapacityPercent > Config.capacity_used_percent_high_water ? 1.0
-                        : (2 * usedCapacityPercent - 0.5));
-        loadScore.replicaNumCoefficient = 1 - loadScore.capacityCoefficient;
+        if (!Config.be_rebalancer_fuzzy_test) {
+            loadScore.capacityCoefficient = usedCapacityPercent < 0.5 ? 0.5
+                    : (usedCapacityPercent > Config.capacity_used_percent_high_water ? 1.0
+                            : (2 * usedCapacityPercent - 0.5));
+            loadScore.replicaNumCoefficient = 1 - loadScore.capacityCoefficient;
+        }
         loadScore.score = capacityProportion * loadScore.capacityCoefficient
                 + replicaNumProportion * loadScore.replicaNumCoefficient;
-        
+
         return loadScore;
     }
 
@@ -315,6 +318,60 @@ public class BackendLoadStatistic {
         return status;
     }
 
+    /**
+     * Check whether the backend can be more balance if we migrate a tablet with size 'tabletSize' from
+     * `srcPath` to 'destPath'
+     * 1. recalculate the load score of src and dest path after migrate the tablet.
+     * 2. if the summary of the diff between the new score and average score becomes smaller, we consider it
+     *    as more balance.
+     */
+    public boolean isMoreBalanced(long srcPath, long destPath, long tabletId, long tabletSize,
+                                  TStorageMedium medium) {
+        long totalCapacity = 0;
+        long totalUsedCapacity = 0;
+        RootPathLoadStatistic srcPathStat = null;
+        RootPathLoadStatistic destPathStat = null;
+        for (RootPathLoadStatistic pathStat : pathStatistics) {
+            if (pathStat.getStorageMedium() == medium) {
+                totalCapacity += pathStat.getCapacityB();
+                totalUsedCapacity += pathStat.getUsedCapacityB();
+                if (pathStat.getPathHash() == srcPath) {
+                    srcPathStat = pathStat;
+                } else if (pathStat.getPathHash() == destPath) {
+                    destPathStat = pathStat;
+                }
+            }
+        }
+        if (srcPathStat == null || destPathStat == null) {
+            LOG.info("migrate {}(size: {}) from {} to {} failed, medium: {}, src or dest path stat does not exist.",
+                    tabletId, tabletSize, srcPath, destPath, medium);
+            return false;
+        }
+        double avgUsedPercent = totalCapacity == 0 ? 0.0 : totalUsedCapacity / (double) totalCapacity;
+        double currentSrcPathScore = srcPathStat.getCapacityB() == 0
+                ? 0.0 : srcPathStat.getUsedCapacityB() / (double) srcPathStat.getCapacityB();
+        double currentDestPathScore = destPathStat.getCapacityB() == 0
+                ? 0.0 : destPathStat.getUsedCapacityB() / (double) destPathStat.getCapacityB();
+
+        double newSrcPathScore = srcPathStat.getCapacityB() == 0
+                ? 0.0 : (srcPathStat.getUsedCapacityB() - tabletSize) / (double) srcPathStat.getCapacityB();
+        double newDestPathScore = destPathStat.getCapacityB() == 0
+                ? 0.0 : (destPathStat.getUsedCapacityB() + tabletSize) / (double) destPathStat.getCapacityB();
+
+        double currentDiff = Math.abs(currentSrcPathScore - avgUsedPercent)
+                + Math.abs(currentDestPathScore - avgUsedPercent);
+        double newDiff = Math.abs(newSrcPathScore - avgUsedPercent) + Math.abs(newDestPathScore - avgUsedPercent);
+
+        LOG.debug("after migrate {}(size: {}) from {} to {}, medium: {}, the load score changed."
+                        + " src: {} -> {}, dest: {}->{}, average score: {}. current diff: {}, new diff: {},"
+                        + " more balanced: {}",
+                tabletId, tabletSize, srcPath, destPath, medium, currentSrcPathScore, newSrcPathScore,
+                currentDestPathScore, newDestPathScore, avgUsedPercent, currentDiff, newDiff,
+                (newDiff < currentDiff));
+
+        return newDiff < currentDiff;
+    }
+
     public boolean hasAvailDisk() {
         for (RootPathLoadStatistic rootPathLoadStatistic : pathStatistics) {
             if (rootPathLoadStatistic.getDiskState() == DiskState.ONLINE) {
@@ -326,7 +383,7 @@ public class BackendLoadStatistic {
 
     /**
      * Classify the paths into 'low', 'mid' and 'high',
-     * and skip offline path, and path with different storage medium 
+     * and skip offline path, and path with different storage medium
      */
     public void getPathStatisticByClass(
             Set<Long> low, Set<Long> mid, Set<Long> high, TStorageMedium storageMedium) {
