@@ -17,8 +17,8 @@
 
 package org.apache.doris.clone;
 
-import org.apache.doris.catalog.Catalog;
 import org.apache.doris.catalog.Database;
+import org.apache.doris.catalog.Env;
 import org.apache.doris.catalog.MaterializedIndex;
 import org.apache.doris.catalog.OlapTable;
 import org.apache.doris.catalog.Partition;
@@ -40,6 +40,7 @@ import org.apache.doris.system.Backend;
 import org.apache.doris.system.SystemInfoService;
 import org.apache.doris.task.AgentTaskQueue;
 import org.apache.doris.task.CloneTask;
+import org.apache.doris.task.StorageMediaMigrationTask;
 import org.apache.doris.thrift.TBackend;
 import org.apache.doris.thrift.TFinishTaskRequest;
 import org.apache.doris.thrift.TStatusCode;
@@ -50,7 +51,6 @@ import org.apache.doris.thrift.TTaskType;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
 import com.google.common.collect.Lists;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -65,24 +65,24 @@ import java.util.Set;
  */
 public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     private static final Logger LOG = LogManager.getLogger(TabletSchedCtx.class);
-    
+
     /*
      * SCHED_FAILED_COUNTER_THRESHOLD:
      *    threshold of times a tablet failed to be scheduled
-     *    
+     *
      * MIN_ADJUST_PRIORITY_INTERVAL_MS:
      *    min interval time of adjusting a tablet's priority
-     *    
+     *
      * MAX_NOT_BEING_SCHEDULED_INTERVAL_MS:
      *    max gap time of a tablet NOT being scheduled.
-     *    
+     *
      * These 3 params is for adjusting priority.
      * If a tablet being scheduled failed for more than SCHED_FAILED_COUNTER_THRESHOLD times, its priority
      * will be downgraded. And the interval between adjustment is larger than MIN_ADJUST_PRIORITY_INTERVAL_MS,
      * to avoid being downgraded too soon.
      * And if a tablet is not being scheduled longer than MAX_NOT_BEING_SCHEDULED_INTERVAL_MS, its priority
      * will be upgraded, to avoid starvation.
-     * 
+     *
      */
     private static final int SCHED_FAILED_COUNTER_THRESHOLD = 5;
     private static final long MIN_ADJUST_PRIORITY_INTERVAL_MS = 5 * 60 * 1000L; // 5 min
@@ -108,12 +108,16 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         BALANCE, REPAIR
     }
 
+    public enum BalanceType {
+        BE_BALANCE, DISK_BALANCE
+    }
+
     public enum Priority {
         LOW,
         NORMAL,
         HIGH,
         VERY_HIGH;
-        
+
         // VERY_HIGH can only be downgraded to NORMAL
         // LOW can only be upgraded to HIGH
         public Priority adjust(Priority origPriority, boolean isUp) {
@@ -128,9 +132,9 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                     return isUp ? NORMAL : LOW;
             }
         }
-        
+
     }
-    
+
     public enum State {
         PENDING, // tablet is not being scheduled
         RUNNING, // tablet is being scheduled
@@ -139,8 +143,9 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         TIMEOUT, // task is timeout
         UNEXPECTED // other unexpected errors
     }
-    
+
     private Type type;
+    private BalanceType balanceType;
 
     /*
      * origPriority is the origin priority being set when this tablet being added to scheduler.
@@ -149,17 +154,17 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
      */
     private Priority origPriority;
     private Priority dynamicPriority;
-    
+
     // we change the dynamic priority based on how many times it fails to be scheduled
     private int failedSchedCounter = 0;
     // clone task failed counter
     private int failedRunningCounter = 0;
-    
+
     // last time this tablet being scheduled
     private long lastSchedTime = 0;
     // last time the dynamic priority being adjusted
     private long lastAdjustPrioTime = 0;
-    
+
     // last time this tablet being visited.
     // being visited means:
     // 1. being visited in TabletScheduler.schedulePendingTablets()
@@ -171,10 +176,10 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
 
     // an approximate timeout of this task, only be set when sending clone task.
     private long taskTimeoutMs = 0;
-    
+
     private State state;
     private TabletStatus tabletStatus;
-    
+
     private String cluster;
     private long dbId;
     private long tblId;
@@ -183,22 +188,27 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     private long tabletId;
     private int schemaHash;
     private TStorageMedium storageMedium;
-    
+
     private long createTime = -1;
     private long finishedTime = -1;
-    
+
     private Tablet tablet = null;
     private long visibleVersion = -1;
     private long committedVersion = -1;
-    
+
     private Replica srcReplica = null;
     private long srcPathHash = -1;
+    // for disk balance to keep src path, and avoid take slot on selectAlternativeTabletsForCluster
+    private Replica tempSrcReplica = null;
     private long destBackendId = -1;
     private long destPathHash = -1;
+    // for disk balance to set migration task's datadir
+    private String destPath = null;
     private String errMsg = null;
-    
+
     private CloneTask cloneTask = null;
-    
+    private StorageMediaMigrationTask storageMediaMigrationTask = null;
+
     // statistics gathered from clone task report
     // the total size of clone files and the total cost time in ms.
     private long copySize = 0;
@@ -224,9 +234,10 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         this.indexId = idxId;
         this.tabletId = tabletId;
         this.createTime = createTime;
-        this.infoService = Catalog.getCurrentSystemInfo();
+        this.infoService = Env.getCurrentSystemInfo();
         this.state = State.PENDING;
         this.replicaAlloc = replicaAlloc;
+        this.balanceType = BalanceType.BE_BALANCE;
     }
 
     public ReplicaAllocation getReplicaAlloc() {
@@ -249,10 +260,18 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         return type;
     }
 
+    public void setBalanceType(BalanceType type) {
+        this.balanceType = type;
+    }
+
+    public BalanceType getBalanceType() {
+        return balanceType;
+    }
+
     public Priority getOrigPriority() {
         return origPriority;
     }
-    
+
     public void setOrigPriority(Priority origPriority) {
         this.origPriority = origPriority;
         // reset dynamic priority along with the origin priority being set.
@@ -261,15 +280,15 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         this.lastSchedTime = 0;
         this.lastAdjustPrioTime = 0;
     }
-    
+
     public Priority getDynamicPriority() {
         return dynamicPriority;
     }
-    
+
     public void increaseFailedSchedCounter() {
         ++failedSchedCounter;
     }
-    
+
     public int getFailedSchedCounter() {
         return failedSchedCounter;
     }
@@ -277,15 +296,15 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public void increaseFailedRunningCounter() {
         ++failedRunningCounter;
     }
-    
+
     public int getFailedRunningCounter() {
         return failedRunningCounter;
     }
-    
+
     public void setLastSchedTime(long lastSchedTime) {
         this.lastSchedTime = lastSchedTime;
     }
-    
+
     public void setLastVisitedTime(long lastVisitedTime) {
         this.lastVisitedTime = lastVisitedTime;
     }
@@ -297,15 +316,15 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public State getState() {
         return state;
     }
-    
+
     public void setState(State state) {
         this.state = state;
     }
-    
+
     public void setTabletStatus(TabletStatus tabletStatus) {
         this.tabletStatus = tabletStatus;
     }
-    
+
     public TabletStatus getTabletStatus() {
         return tabletStatus;
     }
@@ -313,23 +332,23 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public long getDbId() {
         return dbId;
     }
-    
+
     public long getTblId() {
         return tblId;
     }
-    
+
     public long getPartitionId() {
         return partitionId;
     }
-    
+
     public long getIndexId() {
         return indexId;
     }
-    
+
     public long getTabletId() {
         return tabletId;
     }
-    
+
     public void setSchemaHash(int schemaHash) {
         this.schemaHash = schemaHash;
     }
@@ -337,7 +356,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public int getSchemaHash() {
         return schemaHash;
     }
-    
+
     public void setStorageMedium(TStorageMedium storageMedium) {
         this.storageMedium = storageMedium;
     }
@@ -345,23 +364,23 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public TStorageMedium getStorageMedium() {
         return storageMedium;
     }
-    
+
     public String getCluster() {
         return cluster;
     }
-    
+
     public long getCreateTime() {
         return createTime;
     }
-    
+
     public long getCommittedVersion() {
         return visibleVersion;
     }
-    
+
     public void setTablet(Tablet tablet) {
         this.tablet = tablet;
     }
-    
+
     public Tablet getTablet() {
         return tablet;
     }
@@ -370,25 +389,30 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public List<Replica> getReplicas() {
         return tablet.getReplicas();
     }
-    
+
     public void setVersionInfo(long visibleVersion, long committedVersion) {
         this.visibleVersion = visibleVersion;
         this.committedVersion = committedVersion;
     }
-    
+
     public void setDest(Long destBeId, long destPathHash) {
         this.destBackendId = destBeId;
         this.destPathHash = destPathHash;
     }
-    
+
+    public void setDest(Long destBeId, long destPathHash, String destPath) {
+        setDest(destBeId, destPathHash);
+        this.destPath = destPath;
+    }
+
     public void setErrMsg(String errMsg) {
         this.errMsg = errMsg;
     }
-    
+
     public CloneTask getCloneTask() {
         return cloneTask;
     }
-    
+
     public long getCopySize() {
         return copySize;
     }
@@ -414,6 +438,24 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         this.srcPathHash = srcReplica.getPathHash();
     }
 
+    public void setTempSrc(Replica srcReplica) {
+        this.tempSrcReplica = srcReplica;
+    }
+
+    public long getTempSrcBackendId() {
+        if (tempSrcReplica != null) {
+            return tempSrcReplica.getBackendId();
+        }
+        return -1;
+    }
+
+    public long getTempSrcPathHash() {
+        if (tempSrcReplica != null) {
+            return tempSrcReplica.getPathHash();
+        }
+        return -1;
+    }
+
     public long getDestBackendId() {
         return destBackendId;
     }
@@ -422,6 +464,10 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         return destPathHash;
     }
 
+    public String getDestPath() {
+        return destPath;
+    }
+
     // database lock should be held.
     public long getTabletSize() {
         long max = Long.MIN_VALUE;
@@ -432,12 +478,12 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         }
         return max;
     }
-    
+
     /*
-     * check if existing replicas are on same BE.
+     * check if existing replicas are on same BE or Host.
      * database lock should be held.
      */
-    public boolean containsBE(long beId) {
+    public boolean filterDestBE(long beId) {
         Backend backend = infoService.getBackend(beId);
         if (backend == null) {
             // containsBE() is currently only used for choosing dest backend to do clone task.
@@ -451,18 +497,17 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 // BE has been dropped, skip it
                 continue;
             }
-            if (host.equals(be.getHost())) {
+            if (!Config.allow_replica_on_same_host && !FeConstants.runningUnitTest && host.equals(be.getHost())) {
                 return true;
             }
-            // actually there is no need to check BE id anymore, because if hosts are not same, BE ids are
-            // not same either. But for psychological comfort, leave this check here.
+
             if (replica.getBackendId() == beId) {
                 return true;
             }
         }
         return false;
     }
-    
+
     public void setColocateGroupBackendIds(Set<Long> backendsSet) {
         this.colocateBackendsSet = backendsSet;
     }
@@ -481,28 +526,27 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
 
     public boolean compactionRecovered() {
         Replica chosenReplica = null;
-        long maxVersionCount = -1;
-        long minVersionCount = Integer.MAX_VALUE;
+        long maxVersionCount = Integer.MIN_VALUE;
         for (Replica replica : tablet.getReplicas()) {
             if (replica.getVersionCount() > maxVersionCount) {
                 maxVersionCount = replica.getVersionCount();
                 chosenReplica = replica;
             }
-            if (replica.getVersionCount() < minVersionCount) {
-                minVersionCount = replica.getVersionCount();
-            }
         }
         boolean recovered = false;
         for (Replica replica : tablet.getReplicas()) {
-            if (replica.isAlive() && replica.tooSlow() && !chosenReplica.equals(replica)) {
-                chosenReplica.setState(ReplicaState.NORMAL);
-                recovered = true;
+            if (replica.isAlive() && replica.tooSlow() && (!replica.equals(chosenReplica)
+                    || replica.getVersionCount() < Config.min_version_count_indicate_replica_compaction_too_slow)) {
+                if (chosenReplica != null) {
+                    chosenReplica.setState(ReplicaState.NORMAL);
+                    recovered = true;
+                }
             }
         }
         return recovered;
     }
 
-    // database lock should be held.
+    // table lock should be held.
     // If exceptBeId != -1, should not choose src replica with same BE id as exceptBeId
     public void chooseSrcReplica(Map<Long, PathSlot> backendsWorkingSlots, long exceptBeId) throws SchedException {
         /*
@@ -558,7 +602,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         }
         throw new SchedException(Status.SCHEDULE_FAILED, "unable to find source slot");
     }
-    
+
     /*
      * Same rules as choosing source replica for supplement.
      * But we need to check that we can not choose the same replica as dest replica,
@@ -570,14 +614,14 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         Preconditions.checkState(srcReplica.getBackendId() != destBackendId,
                 "wrong be id: " + destBackendId);
     }
-    
+
     /*
      * Rules to choose a destination replica for version incomplete
      * 1. replica's last failed version > 0
      * 2. better to choose a replica which has a lower last failed version
      * 3. best to choose a replica if its last success version > last failed version
      * 4. if these is replica which need further repair, choose that replica.
-     * 
+     *
      * database lock should be held.
      */
     public void chooseDestReplicaForVersionIncomplete(Map<Long, PathSlot> backendsWorkingSlots)
@@ -607,7 +651,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 chosenReplica = replica;
                 break;
             }
-            
+
             if (chosenReplica == null) {
                 chosenReplica = replica;
             } else if (replica.getLastSuccessVersion() > replica.getLastFailedVersion()) {
@@ -618,11 +662,11 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 chosenReplica = replica;
             }
         }
-        
+
         if (chosenReplica == null) {
             throw new SchedException(Status.SCHEDULE_FAILED, "unable to choose dest replica");
         }
-        
+
         // check if the dest replica has available slot
         PathSlot slot = backendsWorkingSlots.get(chosenReplica.getBackendId());
         if (slot == null) {
@@ -639,7 +683,8 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             // So if this replica was previously set to DECOMMISSION, this state needs to be reset to NORMAL.
             // It may happen as follows:
             // 1. A tablet of colocation table is in COLOCATION_REDUNDANT state
-            // 2. The tablet is being scheduled and set one of replica as DECOMMISSION in TabletScheduler.deleteReplicaInternal()
+            // 2. The tablet is being scheduled and set one of replica as
+            //    DECOMMISSION in TabletScheduler.deleteReplicaInternal()
             // 3. The tablet will then be scheduled again
             // 4. But at that time, the BE node of the replica that was
             //    set to the DECOMMISSION state in step 2 is returned to the colocation group.
@@ -649,12 +694,13 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             // forever, because the replica in the DECOMMISSION state will not receive the load task.
             chosenReplica.setWatermarkTxnId(-1);
             chosenReplica.setState(ReplicaState.NORMAL);
-            LOG.info("choose replica {} on backend {} of tablet {} as dest replica for version incomplete," +
-                    " and change state from DECOMMISSION to NORMAL", chosenReplica.getId(), chosenReplica.getBackendId(), tabletId);
+            LOG.info("choose replica {} on backend {} of tablet {} as dest replica for version incomplete,"
+                    + " and change state from DECOMMISSION to NORMAL",
+                    chosenReplica.getId(), chosenReplica.getBackendId(), tabletId);
         }
         setDest(chosenReplica.getBackendId(), chosenReplica.getPathHash());
     }
-    
+
     public void releaseResource(TabletScheduler tabletScheduler) {
         releaseResource(tabletScheduler, false);
     }
@@ -686,12 +732,16 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 }
             }
         }
-        
+
+        if (storageMediaMigrationTask != null) {
+            AgentTaskQueue.removeTask(storageMediaMigrationTask.getBackendId(),
+                    TTaskType.STORAGE_MEDIUM_MIGRATE, storageMediaMigrationTask.getSignature());
+        }
         if (cloneTask != null) {
             AgentTaskQueue.removeTask(cloneTask.getBackendId(), TTaskType.CLONE, cloneTask.getSignature());
 
             // clear all CLONE replicas
-            Database db = Catalog.getCurrentCatalog().getDbNullable(dbId);
+            Database db = Env.getCurrentInternalCatalog().getDbNullable(dbId);
             if (db != null) {
                 Table table = db.getTableNullable(tblId);
                 if (table != null && table.writeLockIfExist()) {
@@ -714,7 +764,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
 
         reset(reserveTablet);
     }
-    
+
     // reset to save memory after state is done
     private void reset(boolean reserveTablet) {
         /*
@@ -729,14 +779,29 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             this.srcPathHash = -1;
             this.destBackendId = -1;
             this.destPathHash = -1;
+            this.destPath = null;
             this.cloneTask = null;
+            this.storageMediaMigrationTask = null;
         }
     }
-    
+
     public void deleteReplica(Replica replica) {
         tablet.deleteReplicaByBackendId(replica.getBackendId());
     }
-    
+
+    public StorageMediaMigrationTask createStorageMediaMigrationTask() throws SchedException {
+        storageMediaMigrationTask = new StorageMediaMigrationTask(getSrcBackendId(), getTabletId(),
+                getSchemaHash(), getStorageMedium());
+        if (destPath == null || destPath.isEmpty()) {
+            throw new SchedException(Status.UNRECOVERABLE,
+                "backend " + srcReplica.getBackendId() + ", dest path is empty");
+        }
+        storageMediaMigrationTask.setDataDir(destPath);
+        this.taskTimeoutMs = getApproximateTimeoutMs();
+        this.state = State.RUNNING;
+        return storageMediaMigrationTask;
+    }
+
     // database lock should be held.
     public CloneTask createCloneReplicaAndTask() throws SchedException {
         Backend srcBe = infoService.getBackend(srcReplica.getBackendId());
@@ -744,13 +809,13 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             throw new SchedException(Status.SCHEDULE_FAILED,
                 "src backend " + srcReplica.getBackendId() + " does not exist");
         }
-        
+
         Backend destBe = infoService.getBackend(destBackendId);
         if (destBe == null) {
             throw new SchedException(Status.SCHEDULE_FAILED,
-                "dest backend " + srcReplica.getBackendId() + " does not exist");
+                "dest backend " + destBackendId + " does not exist");
         }
-        
+
         taskTimeoutMs = getApproximateTimeoutMs();
 
         // create the clone task and clone replica.
@@ -763,25 +828,28 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         // another clone task.
         // That is, we may need to use 2 clone tasks to create a new replica. It is inefficient,
         // but there is no other way now.
-        TBackend tSrcBe = new TBackend(srcBe.getHost(), srcBe.getBePort(), srcBe.getHttpPort());
-        cloneTask = new CloneTask(destBackendId, dbId, tblId, partitionId, indexId,
-                tabletId, schemaHash, Lists.newArrayList(tSrcBe), storageMedium,
-                visibleVersion, (int) (taskTimeoutMs / 1000));
-        cloneTask.setPathHash(srcPathHash, destPathHash);
-        
-        // if this is a balance task, or this is a repair task with REPLICA_MISSING/REPLICA_RELOCATING or REPLICA_MISSING_IN_CLUSTER,
+
+        // if this is a balance task, or this is a repair task with
+        // REPLICA_MISSING/REPLICA_RELOCATING or REPLICA_MISSING_IN_CLUSTER,
         // we create a new replica with state CLONE
         if (tabletStatus == TabletStatus.REPLICA_MISSING || tabletStatus == TabletStatus.REPLICA_MISSING_IN_CLUSTER
                 || tabletStatus == TabletStatus.REPLICA_RELOCATING || type == Type.BALANCE
-                || tabletStatus == TabletStatus.COLOCATE_MISMATCH || tabletStatus == TabletStatus.REPLICA_MISSING_FOR_TAG) {
+                || tabletStatus == TabletStatus.COLOCATE_MISMATCH
+                || tabletStatus == TabletStatus.REPLICA_MISSING_FOR_TAG) {
             Replica cloneReplica = new Replica(
-                    Catalog.getCurrentCatalog().getNextId(), destBackendId,
+                    Env.getCurrentEnv().getNextId(), destBackendId,
                     -1 /* version */, schemaHash,
-                    -1 /* data size */, -1 /* row count */,
+                    -1 /* data size */, -1, -1 /* row count */,
                     ReplicaState.CLONE,
                     committedVersion, /* use committed version as last failed version */
                     -1 /* last success version */);
 
+            TBackend tSrcBe = new TBackend(srcBe.getHost(), srcBe.getBePort(), srcBe.getHttpPort());
+            cloneTask = new CloneTask(destBackendId, dbId, tblId, partitionId, indexId,
+                tabletId, cloneReplica.getId(), schemaHash, Lists.newArrayList(tSrcBe), storageMedium,
+                visibleVersion, (int) (taskTimeoutMs / 1000));
+            cloneTask.setPathHash(srcPathHash, destPathHash);
+
             // addReplica() method will add this replica to tablet inverted index too.
             tablet.addReplica(cloneReplica);
         } else if (tabletStatus == TabletStatus.VERSION_INCOMPLETE) {
@@ -796,12 +864,18 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 throw new SchedException(Status.SCHEDULE_FAILED, "dest replica's path hash is changed. "
                         + "current: " + replica.getPathHash() + ", scheduled: " + destPathHash);
             }
+
+            TBackend tSrcBe = new TBackend(srcBe.getHost(), srcBe.getBePort(), srcBe.getHttpPort());
+            cloneTask = new CloneTask(destBackendId, dbId, tblId, partitionId, indexId,
+                tabletId, replica.getId(), schemaHash, Lists.newArrayList(tSrcBe), storageMedium,
+                visibleVersion, (int) (taskTimeoutMs / 1000));
+            cloneTask.setPathHash(srcPathHash, destPathHash);
         }
-        
+
         this.state = State.RUNNING;
         return cloneTask;
     }
-    
+
     // timeout is between MIN_CLONE_TASK_TIMEOUT_MS and MAX_CLONE_TASK_TIMEOUT_MS
     private long getApproximateTimeoutMs() {
         long tabletSize = getTabletSize();
@@ -810,11 +884,11 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         timeoutMs = Math.min(timeoutMs, Config.max_clone_task_timeout_sec * 1000);
         return timeoutMs;
     }
-    
+
     /*
      * 1. Check if the tablet is already healthy. If yes, ignore the clone task report, and take it as FINISHED.
      * 2. If not, check the reported clone replica, and try to make it effective.
-     * 
+     *
      * Throw SchedException if error happens
      * 1. SCHEDULE_FAILED: will keep the tablet RUNNING.
      * 2. UNRECOVERABLE: will remove the tablet from runningTablets.
@@ -848,9 +922,12 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         }
 
         // 1. check the tablet status first
-        Database db = Catalog.getCurrentCatalog().getDbOrException(dbId, s -> new SchedException(Status.UNRECOVERABLE, "db " + dbId + " does not exist"));
-        OlapTable olapTable = (OlapTable) db.getTableOrException(tblId, s -> new SchedException(Status.UNRECOVERABLE, "tbl " + tabletId + " does not exist"));
-        olapTable.writeLockOrException(new SchedException(Status.UNRECOVERABLE, "table " + olapTable.getName() + " does not exist"));
+        Database db = Env.getCurrentInternalCatalog().getDbOrException(dbId,
+                s -> new SchedException(Status.UNRECOVERABLE, "db " + dbId + " does not exist"));
+        OlapTable olapTable = (OlapTable) db.getTableOrException(tblId,
+                s -> new SchedException(Status.UNRECOVERABLE, "tbl " + tabletId + " does not exist"));
+        olapTable.writeLockOrException(new SchedException(Status.UNRECOVERABLE, "table "
+                + olapTable.getName() + " does not exist"));
         try {
             Partition partition = olapTable.getPartition(partitionId);
             if (partition == null) {
@@ -903,7 +980,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                         "replica does not exist. backend id: " + destBackendId);
             }
 
-            replica.updateVersionInfo(reportedTablet.getVersion(),
+            replica.updateVersionInfo(reportedTablet.getVersion(), reportedTablet.getDataSize(),
                     reportedTablet.getDataSize(), reportedTablet.getRowCount());
             if (reportedTablet.isSetPathHash()) {
                 replica.setPathHash(reportedTablet.getPathHash());
@@ -925,17 +1002,18 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                     reportedTablet.getVersion(),
                     reportedTablet.getSchemaHash(),
                     reportedTablet.getDataSize(),
+                    reportedTablet.getRemoteDataSize(),
                     reportedTablet.getRowCount(),
                     replica.getLastFailedVersion(),
                     replica.getLastSuccessVersion());
 
             if (replica.getState() == ReplicaState.CLONE) {
                 replica.setState(ReplicaState.NORMAL);
-                Catalog.getCurrentCatalog().getEditLog().logAddReplica(info);
+                Env.getCurrentEnv().getEditLog().logAddReplica(info);
             } else {
                 // if in VERSION_INCOMPLETE, replica is not newly created, thus the state is not CLONE
                 // so we keep it state unchanged, and log update replica
-                Catalog.getCurrentCatalog().getEditLog().logUpdateReplica(info);
+                Env.getCurrentEnv().getEditLog().logUpdateReplica(info);
             }
 
             state = State.FINISHED;
@@ -959,25 +1037,25 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             this.copyTimeMs = request.getCopyTimeMs();
         }
     }
-    
+
     /*
      * we try to adjust the priority based on schedule history
      * 1. If failed counter is larger than FAILED_COUNTER_THRESHOLD, which means this tablet is being scheduled
      *    at least FAILED_TIME_THRESHOLD times and all are failed. So we downgrade its priority.
      *    Also reset the failedCounter, or it will be downgraded forever.
-     *    
+     *
      * 2. Else, if it has been a long time since last time the tablet being scheduled, we upgrade its
      *    priority to let it more available to be scheduled.
-     *    
+     *
      * The time gap between adjustment should be larger than MIN_ADJUST_PRIORITY_INTERVAL_MS, to avoid
      * being downgraded too fast.
-     *    
+     *
      * eg:
      *    A tablet has been scheduled for 5 times and all were failed. its priority will be downgraded. And if it is
      *    scheduled for 5 times and all are failed again, it will be downgraded again, until to the LOW.
      *    And than, because of LOW, this tablet can not be scheduled for a long time, and it will be upgraded
      *    to NORMAL, if still not being scheduled, it will be upgraded up to VERY_HIGH.
-     *    
+     *
      * return true if dynamic priority changed
      */
     public boolean adjustPriority(TabletSchedulerStat stat) {
@@ -991,10 +1069,10 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 return false;
             }
         }
-        
+
         boolean isDowngrade = false;
         boolean isUpgrade = false;
-        
+
         if (failedSchedCounter > SCHED_FAILED_COUNTER_THRESHOLD) {
             isDowngrade = true;
         } else {
@@ -1003,14 +1081,14 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
                 isUpgrade = true;
             }
         }
-        
+
         Priority originDynamicPriority = dynamicPriority;
         if (isDowngrade) {
             dynamicPriority = dynamicPriority.adjust(origPriority, false /* downgrade */);
             failedSchedCounter = 0;
             if (originDynamicPriority != dynamicPriority) {
                 LOG.debug("downgrade dynamic priority from {} to {}, origin: {}, tablet: {}",
-                    originDynamicPriority.name(), dynamicPriority.name(), origPriority.name(), tabletId);
+                        originDynamicPriority.name(), dynamicPriority.name(), origPriority.name(), tabletId);
                 stat.counterTabletPrioDowngraded.incrementAndGet();
                 return true;
             }
@@ -1019,23 +1097,23 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             // no need to set lastSchedTime, lastSchedTime is set each time we schedule this tablet
             if (originDynamicPriority != dynamicPriority) {
                 LOG.debug("upgrade dynamic priority from {} to {}, origin: {}, tablet: {}",
-                    originDynamicPriority.name(), dynamicPriority.name(), origPriority.name(), tabletId);
+                        originDynamicPriority.name(), dynamicPriority.name(), origPriority.name(), tabletId);
                 stat.counterTabletPrioUpgraded.incrementAndGet();
                 return true;
             }
         }
         return false;
     }
-    
+
     public boolean isTimeout() {
         if (state != TabletSchedCtx.State.RUNNING) {
             return false;
         }
-        
+
         Preconditions.checkState(lastSchedTime != 0 && taskTimeoutMs != 0, lastSchedTime + "-" + taskTimeoutMs);
         return System.currentTimeMillis() - lastSchedTime > taskTimeoutMs;
     }
-    
+
     public List<String> getBrief() {
         List<String> result = Lists.newArrayList();
         result.add(String.valueOf(tabletId));
@@ -1063,7 +1141,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
         result.add(Strings.nullToEmpty(errMsg));
         return result;
     }
-    
+
     /*
      * First compared by dynamic priority. higher priority rank ahead.
      * If priority is equals, compared by last visit time, earlier visit time rank ahead.
@@ -1084,7 +1162,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
             }
         }
     }
-    
+
     @Override
     public String toString() {
         StringBuilder sb = new StringBuilder();
@@ -1128,7 +1206,7 @@ public class TabletSchedCtx implements Comparable<TabletSchedCtx> {
     public void resetReplicaState() {
         if (tablet != null) {
             for (Replica replica : tablet.getReplicas()) {
-                // To address issue: https://github.com/apache/incubator-doris/issues/9422
+                // To address issue: https://github.com/apache/doris/issues/9422
                 // the DECOMMISSION state is set in TabletScheduler and not persist to meta.
                 // So it is reasonable to reset this state if we failed to scheduler this tablet.
                 // That is, if the TabletScheduler cannot process the tablet, then it should reset
