@@ -17,29 +17,28 @@
 
 package org.apache.doris.qe.cache;
 
-import org.apache.doris.analysis.CompoundPredicate;
 import org.apache.doris.analysis.BinaryPredicate;
+import org.apache.doris.analysis.CompoundPredicate;
 import org.apache.doris.analysis.DateLiteral;
-import org.apache.doris.analysis.InPredicate;
-import org.apache.doris.analysis.PartitionValue;
 import org.apache.doris.analysis.Expr;
-import org.apache.doris.analysis.LiteralExpr;
+import org.apache.doris.analysis.InPredicate;
 import org.apache.doris.analysis.IntLiteral;
-import org.apache.doris.catalog.OlapTable;
-import org.apache.doris.catalog.PartitionItem;
-import org.apache.doris.catalog.RangePartitionInfo;
+import org.apache.doris.analysis.LiteralExpr;
+import org.apache.doris.analysis.PartitionValue;
 import org.apache.doris.catalog.Column;
+import org.apache.doris.catalog.OlapTable;
 import org.apache.doris.catalog.Partition;
+import org.apache.doris.catalog.PartitionItem;
 import org.apache.doris.catalog.PartitionKey;
+import org.apache.doris.catalog.RangePartitionInfo;
 import org.apache.doris.catalog.RangePartitionItem;
+import org.apache.doris.catalog.ScalarType;
 import org.apache.doris.catalog.Type;
+import org.apache.doris.common.AnalysisException;
 import org.apache.doris.common.Config;
 import org.apache.doris.planner.PartitionColumnFilter;
 
-import org.apache.doris.common.AnalysisException;
-
 import com.google.common.collect.Lists;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -119,7 +118,7 @@ public class PartitionRange {
             this.tooNew = false;
         }
 
-        public void Debug() {
+        public void debug() {
             if (partition != null) {
                 LOG.info("partition id {}, cacheKey {}, version {}, time {}, fromCache {}, tooNew {} ",
                         partitionId, cacheKey.realValue(),
@@ -177,10 +176,15 @@ public class PartitionRange {
             switch (type.getPrimitiveType()) {
                 case BOOLEAN:
                 case TIME:
+                case TIMEV2:
                 case DATETIME:
+                case DATETIMEV2:
                 case FLOAT:
                 case DOUBLE:
                 case DECIMALV2:
+                case DECIMAL32:
+                case DECIMAL64:
+                case DECIMAL128:
                 case CHAR:
                 case VARCHAR:
                 case STRING:
@@ -188,6 +192,7 @@ public class PartitionRange {
                     LOG.info("PartitionCache not support such key type {}", type.toSql());
                     return false;
                 case DATE:
+                case DATEV2:
                     date = getDateValue(expr);
                     keyType = KeyType.DATE;
                     break;
@@ -198,6 +203,8 @@ public class PartitionRange {
                     value = expr.getLongValue();
                     keyType = KeyType.LONG;
                     break;
+                default:
+                    return true;
             }
             return true;
         }
@@ -208,10 +215,6 @@ public class PartitionRange {
             date = key.date;
         }
 
-        public boolean equals(PartitionKeyType key) {
-            return realValue() == key.realValue();
-        }
-
         public void add(int num) {
             if (keyType == KeyType.DATE) {
                 date = new Date(date.getTime() + num * 3600 * 24 * 1000);
@@ -244,6 +247,7 @@ public class PartitionRange {
             try {
                 dt = df8.parse(String.valueOf(value));
             } catch (Exception e) {
+                // CHECKSTYLE IGNORE THIS LINE
             }
             return dt;
         }
@@ -452,7 +456,7 @@ public class PartitionRange {
                 LiteralExpr newLiteral;
                 if (key.keyType == KeyType.DATE) {
                     try {
-                        newLiteral = new DateLiteral(key.toString(), Type.DATE);
+                        newLiteral = new DateLiteral(key.toString(), ScalarType.getDefaultDateType(Type.DATE));
                     } catch (Exception e) {
                         LOG.warn("Date's format is error {},{}", key.toString(), e);
                         continue;
@@ -562,7 +566,6 @@ public class PartitionRange {
             return null;
         }
         PartitionColumnFilter partitionColumnFilter = new PartitionColumnFilter();
-        ;
         for (Expr expr : partitionKeyPredicate.getChildren()) {
             if (expr instanceof BinaryPredicate) {
                 BinaryPredicate binPredicate = (BinaryPredicate) expr;
