@@ -18,23 +18,25 @@
 package org.apache.doris.system;
 
 import org.apache.doris.alter.DecommissionType;
-import org.apache.doris.catalog.Catalog;
 import org.apache.doris.catalog.DiskInfo;
 import org.apache.doris.catalog.DiskInfo.DiskState;
+import org.apache.doris.catalog.Env;
+import org.apache.doris.common.Config;
 import org.apache.doris.common.FeConstants;
 import org.apache.doris.common.io.Text;
 import org.apache.doris.common.io.Writable;
+import org.apache.doris.common.util.PrintableMap;
 import org.apache.doris.persist.gson.GsonUtils;
 import org.apache.doris.resource.Tag;
 import org.apache.doris.system.HeartbeatResponse.HbStatus;
 import org.apache.doris.thrift.TDisk;
 import org.apache.doris.thrift.TStorageMedium;
 
+import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.gson.annotations.SerializedName;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 
@@ -44,6 +46,8 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
+import java.util.Optional;
 import java.util.concurrent.atomic.AtomicBoolean;
 
 /**
@@ -116,8 +120,26 @@ public class Backend implements Writable {
     // additional backendStatus information for BE, display in JSON format
     @SerializedName("backendStatus")
     private BackendStatus backendStatus = new BackendStatus();
-    @SerializedName("tag")
-    private Tag tag = Tag.DEFAULT_BACKEND_TAG;
+    // the locationTag is also saved in tagMap, use a single field here to avoid
+    // creating this everytime we get it.
+    @SerializedName(value = "locationTag", alternate = {"tag"})
+    private Tag locationTag = Tag.DEFAULT_BACKEND_TAG;
+
+    @SerializedName("nodeRole")
+    private Tag nodeRoleTag = Tag.DEFAULT_NODE_ROLE_TAG;
+
+    // tag type -> tag value.
+    // A backend can only be assigned to one tag type, and each type can only have one value.
+    @SerializedName("tagMap")
+    private Map<String, String> tagMap = Maps.newHashMap();
+
+    // Counter of heartbeat failure.
+    // Once a heartbeat failed, increase this counter by one.
+    // And if it reaches Config.max_backend_heartbeat_failure_tolerance_count, this backend
+    // will be marked as dead.
+    // And once it back to alive, reset this counter.
+    // No need to persist, because only master FE handle heartbeat.
+    private int heartbeatFailureCounter = 0;
 
     public Backend() {
         this.host = "";
@@ -135,6 +157,7 @@ public class Backend implements Writable {
         this.ownerClusterName = "";
         this.backendState = BackendState.free.ordinal();
         this.decommissionType = DecommissionType.SystemDecommission.ordinal();
+        this.tagMap.put(locationTag.type, locationTag.value);
     }
 
     public Backend(long id, String host, int heartbeatPort) {
@@ -155,6 +178,7 @@ public class Backend implements Writable {
         this.ownerClusterName = "";
         this.backendState = BackendState.free.ordinal();
         this.decommissionType = DecommissionType.SystemDecommission.ordinal();
+        this.tagMap.put(locationTag.type, locationTag.value);
     }
 
     public long getId() {
@@ -322,6 +346,10 @@ public class Backend implements Writable {
         return backendStatus;
     }
 
+    public int getHeartbeatFailureCounter() {
+        return heartbeatFailureCounter;
+    }
+
     /**
      * backend belong to some cluster
      *
@@ -396,6 +424,17 @@ public class Backend implements Writable {
         return dataUsedCapacityB;
     }
 
+    public long getRemoteUsedCapacityB() {
+        ImmutableMap<String, DiskInfo> disks = disksRef;
+        long totalRemoteUsedCapacityB = 0L;
+        for (DiskInfo diskInfo : disks.values()) {
+            if (diskInfo.getState() == DiskState.ONLINE) {
+                totalRemoteUsedCapacityB += diskInfo.getRemoteUsedCapacity();
+            }
+        }
+        return totalRemoteUsedCapacityB;
+    }
+
     public double getMaxDiskUsedPct() {
         ImmutableMap<String, DiskInfo> disks = disksRef;
         double maxPct = 0.0;
@@ -417,7 +456,8 @@ public class Backend implements Writable {
         ImmutableMap<String, DiskInfo> diskInfos = disksRef;
         boolean exceedLimit = true;
         for (DiskInfo diskInfo : diskInfos.values()) {
-            if (diskInfo.getState() == DiskState.ONLINE && diskInfo.getStorageMedium() == storageMedium && !diskInfo.exceedLimit(true)) {
+            if (diskInfo.getState() == DiskState.ONLINE && diskInfo.getStorageMedium()
+                    == storageMedium && !diskInfo.exceedLimit(true)) {
                 exceedLimit = false;
                 break;
             }
@@ -462,7 +502,7 @@ public class Backend implements Writable {
             }
             if (allPathHashUpdated) {
                 initPathInfo = true;
-                Catalog.getCurrentSystemInfo().updatePathInfo(new ArrayList<>(disks.values()), Lists.newArrayList());
+                Env.getCurrentSystemInfo().updatePathInfo(new ArrayList<>(disks.values()), Lists.newArrayList());
             }
         }
 
@@ -494,6 +534,10 @@ public class Backend implements Writable {
             diskInfo.setTotalCapacityB(totalCapacityB);
             diskInfo.setDataUsedCapacityB(dataUsedCapacityB);
             diskInfo.setAvailableCapacityB(diskAvailableCapacityB);
+            if (tDisk.isSetRemoteUsedCapacity()) {
+                diskInfo.setRemoteUsedCapacity(tDisk.getRemoteUsedCapacity());
+            }
+
             if (tDisk.isSetPathHash()) {
                 diskInfo.setPathHash(tDisk.getPathHash());
             }
@@ -527,15 +571,35 @@ public class Backend implements Writable {
         if (isChanged) {
             // update disksRef
             disksRef = ImmutableMap.copyOf(newDiskInfos);
-            Catalog.getCurrentSystemInfo().updatePathInfo(addedDisks, removedDisks);
+            Env.getCurrentSystemInfo().updatePathInfo(addedDisks, removedDisks);
             // log disk changing
-            Catalog.getCurrentCatalog().getEditLog().logBackendStateChange(this);
+            Env.getCurrentEnv().getEditLog().logBackendStateChange(this);
+        }
+    }
+
+    /**
+     * In old version, there is only one tag for a Backend, and it is a "location" type tag.
+     * But in new version, a Backend can have multi tag, so we need to put locationTag to
+     * the new tagMap
+     */
+    private void convertToTagMapAndSetLocationTag() {
+        if (tagMap == null) {
+            // When first upgrade from old version, tags may be null
+            tagMap = Maps.newHashMap();
+        }
+        if (!locationTag.value.equals(tagMap.get(Tag.TYPE_LOCATION))) {
+            // ATTN: here we use Tag.TYPE_LOCATION directly, not locationTag.type,
+            // because we need to make sure the previous tag must be a location type tag,
+            // and if not, convert it to location type.
+            tagMap.put(Tag.TYPE_LOCATION, locationTag.value);
         }
     }
 
     public static Backend read(DataInput in) throws IOException {
         String json = Text.readString(in);
-        return GsonUtils.GSON.fromJson(json, Backend.class);
+        Backend be = GsonUtils.GSON.fromJson(json, Backend.class);
+        be.convertToTagMapAndSetLocationTag();
+        return be;
     }
 
     @Override
@@ -544,30 +608,9 @@ public class Backend implements Writable {
         Text.writeString(out, json);
     }
 
-    @Deprecated
-    private void readFields(DataInput in) throws IOException {
-        id = in.readLong();
-        host = Text.readString(in);
-        heartbeatPort = in.readInt();
-        bePort = in.readInt();
-        httpPort = in.readInt();
-        beRpcPort = in.readInt();
-        isAlive.set(in.readBoolean());
-        isDecommissioned.set(in.readBoolean());
-        lastUpdateMs = in.readLong();
-        lastStartTime = in.readLong();
-        Map<String, DiskInfo> disks = Maps.newHashMap();
-        int size = in.readInt();
-        for (int i = 0; i < size; i++) {
-            String rootPath = Text.readString(in);
-            DiskInfo diskInfo = DiskInfo.read(in);
-            disks.put(rootPath, diskInfo);
-        }
-        disksRef = ImmutableMap.copyOf(disks);
-        ownerClusterName = Text.readString(in);
-        backendState = in.readInt();
-        decommissionType = in.readInt();
-        brpcPort = in.readInt();
+    @Override
+    public int hashCode() {
+        return Objects.hash(id, host, heartbeatPort, bePort, isAlive);
     }
 
     @Override
@@ -588,7 +631,7 @@ public class Backend implements Writable {
     @Override
     public String toString() {
         return "Backend [id=" + id + ", host=" + host + ", heartbeatPort=" + heartbeatPort + ", alive=" + isAlive.get()
-                + ", tag: " + tag + "]";
+                + ", tags: " + tagMap + "]";
     }
 
     public String getOwnerClusterName() {
@@ -652,6 +695,12 @@ public class Backend implements Writable {
                 this.brpcPort = hbResponse.getBrpcPort();
             }
 
+            if (!this.getNodeRoleTag().value.equals(hbResponse.getNodeRole()) && Tag.validNodeRoleTag(
+                    hbResponse.getNodeRole())) {
+                isChanged = true;
+                this.nodeRoleTag = Tag.createNotCheck(Tag.TYPE_ROLE, hbResponse.getNodeRole());
+            }
+
             this.lastUpdateMs = hbResponse.getHbTime();
             if (!isAlive.get()) {
                 isChanged = true;
@@ -663,12 +712,19 @@ public class Backend implements Writable {
             }
 
             heartbeatErrMsg = "";
+            this.heartbeatFailureCounter = 0;
         } else {
-            if (isAlive.compareAndSet(true, false)) {
-                isChanged = true;
-                LOG.warn("{} is dead,", this.toString());
+            // Only set backend to dead if the heartbeat failure counter exceed threshold.
+            if (++this.heartbeatFailureCounter >= Config.max_backend_heartbeat_failure_tolerance_count) {
+                if (isAlive.compareAndSet(true, false)) {
+                    isChanged = true;
+                    LOG.warn("{} is dead,", this.toString());
+                }
             }
 
+            // still set error msg and missing time even if we may not mark this backend as dead,
+            // for debug easily.
+            // But notice that if isChanged = false, these msg will not sync to other FE.
             heartbeatErrMsg = hbResponse.getMsg() == null ? "Unknown error" : hbResponse.getMsg();
             lastMissingHeartbeatTime = System.currentTimeMillis();
         }
@@ -711,12 +767,49 @@ public class Backend implements Writable {
         public volatile boolean isLoadDisabled = false;
     }
 
-    public void setTag(Tag tag) {
-        this.tag = tag;
+    public Tag getLocationTag() {
+        return locationTag;
     }
 
-    public Tag getTag() {
-        return tag;
+    public Tag getNodeRoleTag() {
+        return nodeRoleTag;
+    }
+
+    public boolean isMixNode() {
+        return nodeRoleTag.value.equals(Tag.VALUE_MIX);
+    }
+
+    public boolean isComputeNode() {
+        return nodeRoleTag.value.equals(Tag.VALUE_COMPUTATION);
+    }
+
+    public void setTagMap(Map<String, String> tagMap) {
+        Preconditions.checkState(tagMap.containsKey(Tag.TYPE_LOCATION));
+        this.tagMap = tagMap;
+        this.locationTag = Tag.createNotCheck(Tag.TYPE_LOCATION, tagMap.get(Tag.TYPE_LOCATION));
+        if (tagMap.containsKey(Tag.TYPE_ROLE) && Tag.validNodeRoleTag(tagMap.get(Tag.TYPE_ROLE))) {
+            this.nodeRoleTag = Tag.createNotCheck(Tag.TYPE_ROLE, tagMap.get(Tag.TYPE_ROLE));
+        }
+    }
+
+    public Map<String, String> getTagMap() {
+        return tagMap;
+    }
+
+    public String getTagMapString() {
+        return "{" + new PrintableMap<>(tagMap, ":", true, false).toString() + "}";
+    }
+
+    /**
+     * Get Tag by type, return Optional.empty if no such tag with given type
+     *
+     * @param type
+     * @return
+     */
+    public Optional<Tag> getTagByType(String type) {
+        if (!tagMap.containsKey(type)) {
+            return Optional.empty();
+        }
+        return Optional.of(Tag.createNotCheck(type, tagMap.get(type)));
     }
 }
-
